/*
 * This is a proposal of a replacement to std.h. It is used with
 * 'inherit "room/room";'.
 * All global variables below are supposed to be setup by the reset()
 * in the function that inherits this file.
 */

/*
 * Additions by Ronny Wikh 910711.
 *
 * Optional third argument in dest_dir for each exit. If it exists it
 * must be 0 or a string "#exit_function". This function is called before 
 * each move and the move is cancelled if it returns 1.
 * The destination room as given in the corresponding first argument in 
 * dest_dir is given as argument to this function.
 *
 * skip_obvious - If this variable is set, the text "There are nn obvious..." 
 * is not shown.
 */

/* Addition by Travis: 190192.
 * changed short(), replaced it with Exodor's version.
 */

/* The variable that controls the printing of obvious exits. */
int skip_obvious;

/* An array with destinations and directions: "room/church", "north" ... */
string dest_dir;

/* Short description of the room */
string short_desc;

/* Long description of the room */
string long_desc;

/* Special items in the room. "table", "A nice table", "window", "A window" */
string items;

/* Fact about this room. ex: "no_fight", "no_steal" */
string property;

/* No castles are allowed to be dropped here */
int no_castle_flag;


init() 
{
    int i, n;
    string func;

    if (!dest_dir)
	return;
    n = sizeof(dest_dir);
    if (n > 2)
    {
        func = dest_dir[2];
	
	if (!func)
	    n = 3;
	else
	    if(func[0] == 35)
	        n = 3;
	    else 
	        n = 2;
    }
    else
        n = 2;

    while(i < sizeof(dest_dir)) 
    {
	add_action("move", dest_dir[i + 1]);
	i += n;
    }
}

id(str) 
{
    int i;
    if (!items)
	return;
    while(i < sizeof(items)) 
    {
	if (items[i] == str)
	    return 1;
	i += 2;
    }
    return 0;
}

long(str) 
{
    int i, n;
    string func;

    if (set_light(0) == 0)
    {
	write("It is too dark.\n");
	return;
    }
    if (!str) 
    {
	write(long_desc);
	if(!skip_obvious)
	{
	    if (!dest_dir)
	    {
  	        write("    No obvious exits.\n");
		return;
	    }

	    n = sizeof(dest_dir);
	    if (n > 2)
	    {
	        func = dest_dir[2];
		
	        if (!func)
		    n = 3;
		else
		    if(func[0] == 35)
		        n = 3;
		    else 
		        n = 2;
	    }
	    else
	        n = 2;

	    i = 1;
	    if (sizeof(dest_dir) == n)
	        write("    There is one obvious exit:");
	    else
	        write("    There are " + convert_number(sizeof(dest_dir) / n) +
		      " obvious exits:");
	    while(i < sizeof(dest_dir)) 
	    {
	        write(" " + dest_dir[i]);
		i += n;
		if (i == sizeof(dest_dir) - 1)
		    write(" and");
		else if (i < sizeof(dest_dir))
		    write(",");
	    }
	    write("\n");
	}
	return;
    }

    if (!items)
	return;
    i = 0;
    while(i < sizeof(items)) 
    {
	if (items[i] == str) 
	{
	    write(items[i+1] + ".\n");
	    return;
	}
	i += 2;
    }
}

/*
 * Does this room has a special property ?
 * The 'property' variable can be both a string and array of strings.
 * If no argument is given, return the 'property' variable.
 */
query_property(str) 
{
    int i;
    if (str == 0)
	return property;
    if (!property)
	return 0;
    if (stringp(property))
	return str == property;
    while(i < sizeof(property)) 
    {
	if (property[i] == str)
	    return 1;
	i += 1;
    }
    return 0;
}

move(str) 
{
    int i, n;
    string func;

    n = sizeof(dest_dir);
    if (n > 2)
    {
        func = dest_dir[2];
	if (!func)
	    n = 3;
	else
	    if (func[0] == 35)
	        n = 3;
	    else 
  	        n = 2;
    }
    else
	n = 2;

    while (i < sizeof(dest_dir))
    {
	if (query_verb() == dest_dir[i + 1])
	{
	    if (n == 3 && dest_dir[i + 2] != 0)
	    {
		if (!sscanf(dest_dir[i + 2], "#%s", func))
		    return;
		if (call_other(this_object(), func, dest_dir[i]))
		    return 1;
	    }
	    this_player()->move_player(dest_dir[i + 1] + "#" + dest_dir[i]);
	    return 1;
	}
	i += n;
    }
}

# if 0
short() 
{
    if (set_light(0))
	return short_desc;
    return "It is too dark";
}
# else  /*** Code by Exodor, patched by Zappa. ***/
        /*** Supports 3 args now in dest_dir - Travis ***/
short()
{
string shrt, func;
int i, n;

   if (set_light(0))
   {
      shrt = short_desc;
      if (!skip_obvious)
      {
         shrt += " (";
         if (dest_dir)
         {

            n = sizeof(dest_dir);
            if (n > 2)
            {
                func = dest_dir[2];

                if (!func)
                    n = 3;
                else if(func[0] == 35)
                    n = 3;
                else
                    n = 2;
            }
            else n = 2;

            for(i=1; i<sizeof(dest_dir); i+=n) {
                shrt+= extract(dest_dir[i],0,0);
                if(i+n<sizeof(dest_dir)) shrt+= ",";
            }
         }
         shrt+=")";
      }
      return shrt;
   }
   return "It is too dark";
}
# endif

query_dest_dir() 
{
    return dest_dir;
}

query_long() 
{
    return long_desc;
}

/*
 * Convert a number to a word. The array is being created by the
 * standard room/room, and shared by all rooms.
 */
string numbers;

convert_number(n) 
{
    if (!pointerp(numbers))
	numbers = query_numbers();
    if (n > 9)
	return "lots of";
    return numbers[n];
}

query_numbers() 
{
    if (!numbers) 
    {
	if (file_name(this_object()) == "room/room")
	    numbers = ({"no", "one", "two", "three", "four", "five",
			    "six", "seven", "eight", "nine" });
	else
	    numbers = call_other("room/room", "query_numbers");
    }
    return numbers;
}

query_drop_castle() 
{
    return no_castle_flag;
}

