
/*
 *         Stdlib.c
 *
 *         Inherit-file for standard 'commonly' used functions
 *         Author: Brainiac (pimentel@fwi.uva.nl)
 *         Last mod: 4-9-91
 *
 *         Calls included in this package:
 *           - me()
 *           - my_location()
 *           - my_name()
 *           - my_level()
 *           - me_interactive()
 *           - inventory()
 *           - player_exists(name) 
 *
 *         You are encouraged by the author to use this inherit-package.
 *         Copying/mutating it, however, without the author's explicit
 *         permission is prohibited.
 *                       
 *         (C) by Brainiac (Andy Pimentel), 1991.
 */

#include "lib.h"

inherit "/lib/list";

/****************************************************************************
 * Commonly used calls on this_player()
 ****************************************************************************/

me()
{
   return this_player();
}

my_location()
{
   return (!me()) ? 0 : environment(this_player());
}

my_name()
{
   return (!me()) ? 0 : capitalize(this_player()->query_real_name());
}

my_level()
{
   return (!me()) ? 0 : this_player()->query_level();
}

me_interactive()
{
   return (!me()) ? 0 : query_ip_number(this_player());
}

/****************************************************************************/

player_exists(name)
/*
 * Checks whether the player with name 'name' exists or not
 * Syntax: status player_exists(string)
 * Returns true when exists, else false.
 */
{
   return restore_object("users/" + name);
}

inventory(obj)
/*
 * Get the inventory of an object
 * Syntax: array inventory(object)
 * Returns the inventory of the object in an array.
 */
{
    object inv, item;
    int i;

    if (!obj || !objectp(obj))
    {
       write("Bad argument to inventory()\n");
       return 1;
    }
    for (item = first_inventory(obj); item && i < MAX_ELTS; i++)
    {
       inv = add_to_list(item, inv);
       item = next_inventory(item);
    }
    return inv;
}

