
/*
 *         String.c
 *
 *         Inherit-file for string-manipulation
 *         Author: Brainiac (pimentel@fwi.uva.nl)
 *         Last mod: 7-9-91
 *
 *         Calls included in this package:
 *           - strip_newline(string)
 *           - write_at(pos, string)
 *           - str_out(string, substring)
 *           - sti(string)
 *           - up(string)
 *           - parse_string(string)
 *           - format_string(array, int);		
 *
 *         You are encouraged by the author to use this inherit-package.
 *         Copying/mutating it, however, without the author's explicit
 *         permission is prohibited.
 *                       
 *         (C) by Brainiac (Andy Pimentel), 1991.
 */

#include "lib.h"

strip_newline(str)
/*
 * Strips the newline at the end of a string
 * Syntax: string strip_newline(string)
 * Returns new skipped string.
 */
{
   if (!str || !stringp(str))
   {
      write("Bad argument to strip_newline()\n");
      return 1;
   }
   if (str[strlen(str) - 1] == NEW_LINE)
      return (strlen(str) > 1) ? extract(str, 0, strlen(str) - 2) : "";
   else return str;
}

write_at(pos, str)
/*
 * Writes a string on X coordinate 'pos'
 * Syntax: write_at(int, string)
 */
{
   int i;

   if ((pos && !intp(pos)) || !str || !stringp(str))
   {
      write("Bad argument(s) to write_at()\n");
      return 1;
   }
   if (pos < 0) pos = 0;
   else if (pos > SCREEN_WIDTH) pos = SCREEN_WIDTH;
   for (i = 0; i < pos; i++) write(" ");
   write(str);
}

str_out(source, str)
/*
 * Removes the substring 'str' from 'source' 
 * Syntax: string str_out(string, string)
 * When 'str' present, it returns the new string without 'str', else it
 * returns 0;
 */
{
   string temp1, temp2;
   int n;

   if (!source || !stringp(source) || !str || !stringp(str))
   {
      write("Bad argument(s) to str_out()\n");
      return 0;
   }
   return (!sscanf(source, "%s" + str + "%s", temp1, temp2)) ? 0 : 
							       temp1 + temp2;
}

sti(str) /* String To Integer */
/*
 * Converts a string to an integer
 * Syntax: int sti(string)
 * Returns the integer in case the string contained an integer, else it 
 * will return 0. (beware!)
 */
{
   int i;

   if (!str || !stringp(str))
   {
      write("Bad argument to sti()\n");
      return 0;
   }
   return (!sscanf(str, "%d", i)) ? 0 : i;
}

up(str)
/*
 * Change a string in upper-case characters
 * Syntax: string up(string)
 * Returns the upper-case string.
 */
{
   int i;
   string temp;

   if (!str || !stringp(str))
   {
      write("Bad argument to up()\n");
      return 0;
   }
   temp = "";
   for (i = 0; i < strlen(str); i++) 
      temp = (str[i] >= 'a' && str[i] <= 'z') ?
			temp + capitalize(extract(str, i, i)) :
	                temp + extract(str, i, i);
   return temp;
}

parse_string(str)
/*
 * Parses a string and returns an array with the separate words of the string
 * Syntax: array parse_string(string)
 * Returns an array containing the words of a string.
 */
{
   int curr, pivot, words;
   string ret;
   if (!str || !stringp(str))
   {
      write("Bad argument to parse_string()\n");
      return 0;
   }
   
   do 
   {
      while(curr < strlen(str) &&     /* skip garbage */
           (str[curr] == NEW_LINE || 
	    str[curr] == ' ' ||
	    str[curr] == '.' ||
	    str[curr] == ',' ||
	    str[curr] == TAB))
         curr++;
      if (curr == strlen(str) || words == MAX_ELTS) break; 
      else pivot = curr;
      while(curr < strlen(str) && 
            str[curr] != NEW_LINE &&
	    str[curr] != ' ' &&
	    str[curr] != '.' && 
	    str[curr] != ',' && 
	    str[curr] != TAB)
   	 curr++;
      ret = (!ret) ? ({extract(str, pivot, curr - 1)}) : 
		     ret + ({extract(str, pivot, curr - 1)});
      words++;
   }
   while (curr < strlen(str));
   return ret;
}

format_string(list, offset)
/*
 * Formats a list of strings into a string; each list elt seperated thru
 * a comma, and partitioned over several lines if necessary.
 * Syntax: format_string(array, int)
 * Returns a formatted string.
 */
{
   string s;
   int n, i;

   if (!list) return "";
   if (!pointerp(list) || (offset && !intp(offset)))
   {
      write("Bad argument to format_string()\n");
      return 0;
   }
   if (offset >= SCREEN_WIDTH)
   {
      write("Offset out of range in format_string()\n");
      return 0;
   }
   n = offset;
   s = "";
   for (i = 0; i < sizeof(list); i++)
   {
      if (n + strlen(list[i] + 3) < SCREEN_WIDTH)
	 if (i + 1 < sizeof(list))
	    s += list[i] + ", ";
         else
	    s += list[i] + ".\n";
      else 
      {
	 if (i + 1 < sizeof(list))
	    s += "\n" + list[i] + ", ";
         else
	    s += "\n" + list[i] + ".\n";
	 n = 0;
      }
      n += strlen(list[i]) + 2;
   }

   return s;
}

