/* Groo 91-01-31 */
/**********************************
 * The new, great bulletin board! *
 * (The manic wanderer - version) *
 * (with the nasty bug #1 fixed.) *
 **********************************/

/* Fixed for inheritance by Tintin 91-02-06 */

/* Example of use:
* inherit "obj/bull_base";
* 
* reset(arg)
* {
*       ::reset(arg);
*       if (arg)
*                return;
*       set_board_name("players/tintin/my_bulletin");
*       set_num_notes(10);
*       set_anonymous(1);
*       set_silent(0);
*       set_show_lvl(0);
*       set_remove_lvl(10);
*       set_remove_str("Sorry, you can't do that");
* }
*
*       After setting it up this way you just need to clone it and move it
*       to the location where you want it.
*
*       The following options can be set:
*       set_num_notes(n)        Set max number of notes (default 30)
*       set_silent(n)           0 = Tell room about read and examine of the 
*                               board 1 = silent read & examine,
*                               (default 1)
*       set_anonymous(n)        0 = No anonymous posts, 1 = make articles
*                               made by invisible persons come out as written
*                               by 'Someone' (default 0)
*       set_board_name(name)    Path to the save_file for the board
*                               (no default, this variable MUST be set)
*       set_remove_lvl(n)       Minimum level required to remove other
*                               players notes (default 24)
*       set_remove_str(str)     The string to send to the player if a
*                               remove failed. (default 'Only an Archwizard 
*                               can remove other peoples notes')
*       set_err_log(str)        Name of the log to where error messages
*                               should go. (default 'BOARDS', but you
*                               MUST change it since only /obj and /room
*                               objects can write to this file)
*       set_show_lvl(n) 0 = Don't show level in note header,
*                               1 = show level of writer in note header
*                               (default 1)
*/

string tmp_head, tmp_text, new_head, new_text;
int msg_num;

static string  messages, headers;
static int line, looked_at;
static object curr_writer;
static string board_name, remove_str, err_log;
static int notes, silent, anonymous, remove_lvl, show_lvl;

/*
   These functions are made for setting up the board in the file
   inheriting it.
*/
static  set_num_notes(n) { notes = n; }
static  set_silent(n){ if(n) silent = 1; else silent = 0; }
static  set_anonymous(n){ if(n) anonymous = 1; else anonymous = 0; }
static  set_board_name(str) { board_name = str; }
static  set_remove_lvl(n) { remove_lvl = n; }
static  set_remove_str(str) { remove_str = str; }
static  set_err_log(str) { err_log = str; }
static  set_show_lvl(n) { if (n) show_lvl = 1; else show_lvl = 0;}

/* 
   These functions are hooks that can be replaced with custom functions
   for selecting who may read, write or remove messages in an inherited
   board.
*/
static check_reader() { return; }
static check_writer() { return; }
static check_remove() { return; }

id(str) {
        return str == "board" || str == "bulletin board" || str == "bulletinboard";
}

long() {
        int ind;
        write("This is a bulletin board.\n");
        write("Usage : note <headline>, read/remove <message number>\n");
        write("        store <message number> <file name>\n");
        write("        mread <message number> \n");
        if (check_reader()) {
            return;
        }
        if (!msg_num) {
                write("The board is empty.\n");
                return;
        }
        write("The bulletin board contains " + msg_num);
        if (msg_num == 1)
                write(" note :\n\n");
        else 
                write(" notes :\n\n");
        if (!silent)
                say(this_player()->query_name() + " studies the bulletin board.\n");
        ind = 0;
        while (ind < msg_num) {
                write(ind + 1 + ":\t" + headers[ind] + "\n");
                ind++;
        }
}

short() {
        return "A bulletin board";
}

get() {
        write("It is firmly secured to the ground.\n");
        return 0;
}

init() {
        add_action("new_msg", "note");
        add_action("read_msg", "read");
        add_action("read_msg", "mread");
        add_action("remove_msg", "remove");
        add_action("store_msg", "store");
        if (!looked_at) {
                int i;
                string arr;
                messages = allocate(notes);
                headers = allocate(notes);
                looked_at = 1;
                if (!restore_object(board_name)) {
                        error_log("Unable to restore saved board");
                        write("Board : unable to restore saved messages.\n");
                        return;
                }
                arr = explode(new_head, "\n**\n");
                i = 0;
                while(i < sizeof(arr)) {
                    headers[i] = arr[i];
                    i++;
                }
                arr = explode(new_text, "\n**\n");
                i = 0;
                while(i < sizeof(arr)) {
                    messages[i] = arr[i];
                    i++;
                }
                new_text = "";
                new_head = "";
        }
        return 1;
}

reset(arg) {
        if (arg) {
                return;
        }
        notes = 30;
        board_name = "";
        silent = 1;
        anonymous = 0;
        show_lvl = 1;
        remove_lvl = 24;
        remove_str = "Only an Archwizard can remove other peoples notes.\n";
        err_log = "BOARDS";
}

static new_msg(msg_head) {
        line = 1;
        if (check_writer())
                return 1;
        if (!msg_head)
                return 0;
        if (curr_writer && environment(curr_writer) ==
                environment(this_object())) {
                write(curr_writer -> query_name() + " is busy writing.\n");
                return 1;
        }
        if (msg_num == notes) {
                write("You have to remove an old message first.\n");
                return 1;
        }
        if (strlen(msg_head) > 50) {
                write("Message header too long. Try again.\n");
                return 1;
        }
        curr_writer = this_player();
        say(curr_writer->query_name() + " starts writing a note.\n");
        tmp_head = msg_head + " ";
        tmp_text = "";
        input_to("get_msg");
        write("Enter message text. End with '**', abort with '~q'.\n");
        write("1>>");
        return 1;
}

get_msg(str) {
        if (str == "~q") {
                say(curr_writer->query_name() + " aborts writing a note.\n");
                write("Note aborted.\n");
                curr_writer = 0;
                tmp_head = "";
                tmp_text = "";
                return;
        }
        if (str == "**") {
                if (line == 1) {
                        write("No text entered. Message discarded.\n");
                        say(curr_writer->query_name() + " quits writing.\n");
                        curr_writer = 0;
                        tmp_head = "";
                        tmp_text = "";
                        return;
                }
                say(curr_writer->query_name()+" has completed a note : "+
                        tmp_head + "\n");
                        headers[msg_num] = tmp_head + mk_header();
                messages[msg_num] = tmp_text + "\n";
                msg_num++;
                save_board();
                write("Ok.\n");
                curr_writer = 0;
                return;
        }
        tmp_text = tmp_text + str + "\n";
        line++;
        write(line + ">>");
        input_to("get_msg");
}

static mk_header()
{
        string lvl_text, name;
        lvl_text = "";
        if(show_lvl)
                lvl_text = ", " + this_player() -> query_level();
        if (anonymous)
                name = this_player()->query_name();
        else
                name = capitalize(this_player()->query_real_name());

        return "(" + name + ", " + extract(ctime(time()), 4, 9) + 
                lvl_text + ")";
}
read_msg(what_msg) {
        int note;
        if (check_reader())
                return 1;
        if (!what_msg) {
                write("You read a while but as you can't decide exactly");
                write("what to read, you don't find much of interest.\n");
                return 1;
        }
        if (!sscanf(what_msg, "%d", note))
                if (!sscanf(what_msg, "note %d", note))
                        return 0;
        if (note < 1 || note > msg_num) {
                write("Not that many messages.\n");
                return 1;
        }
        note--;
        if (!silent)
                say(this_player()->query_name()+" reads a note titled '"+
                        headers[note] + "'.\n");

        write("The note is titled '" + headers[note] + "':\n");
        if (query_verb() == "mread")
                mread_msg(note);
        else
                write(messages[note]);
        return 1;
}

static mread_msg(note) {
        object ob;
        ob = clone_object("/obj/boardwalk");
        move_object(ob, this_player());
        ob -> more(messages[note]);
}

static remove_msg(what_msg) {
        string player, title, date, tmp;
        int note, ind, count;
        if (!what_msg)
                return 0;
        if (check_remove())
                return 1;
        if (!sscanf(what_msg, "%d", note)) 
                if (!sscanf(what_msg, "note %d", note))
                        return 0;
        if (note < 1 || note > msg_num) {
                write("That message doesn't exist.\n");
                return 1;
        }
        note--;
        if (sscanf(headers[note], "%s(%s,%s", title, player, date) != 3) {
                error_log("Header error");
                write("Board : error - header corrupt.\n");
                return 1;
        }
        
        /* Get rid of ( the header */
        count = sizeof(explode(player, "("));
        if (count > 1) {
                tmp = explode(player, "(");
                player = tmp[count - 1];
        }
        
        if ((this_player()->query_real_name() != lower_case(player) &&
                this_player()->query_level() < remove_lvl) ||
                !query_ip_number(this_player())) {
                write(remove_str);
                say(this_player()->query_name() + " failed to remove " +
                        "a note.\n");
                return 1;
        }
        say(this_player()->query_name() + " removes a note titled '" +
                headers[note] + "'.\n");
        ind = note;
        while (ind < msg_num - 1) {
                messages[ind] = messages[ind + 1];
                headers[ind] = headers[ind + 1];
                ind++;
        }
        messages[ind] = 0;
        headers[ind] = 0;
        msg_num--;
        save_board();
        write("Ok.\n");
        return 1;
}
static store_msg(str) {
        int note;
        string file;
        if (this_player() -> query_level() < 20) {
                write("Sorry, mortals can't save messages.\n");
                return 1;
        }
        if (check_reader())
                return 1;
        if (!str) 
                return 0;
        if (sscanf(str, "%d %s", note, file) != 2)
                if (sscanf(str, "note %d %s", note, file) != 2)
                        return 0;
        if (note < 1 || note > msg_num) {
                write("That message doesn't exist.\n");
                return 1;
        }
        note--;
        file += ".note";
        write_file(file, headers[note] + "\n" + messages[note] + "\n");
        return 1;
}

static save_board() {
        new_head = implode(headers, "\n**\n") + "\n**\n";
        new_text = implode(messages, "\n**\n") + "\n**\n";
        save_object(board_name);
        new_head = "";
        new_text = "";
        return 1;
}

static error_log(str) {
        tell_room(environment(this_object()), "Board says '" + str + "'.\n");
        log_file(err_log, "Board : " + str + "\n");
        return;
}

query_desc() {
  return "";
}

