/*
 * Finger daemon, by Dworkin
 * v1.0 28-03-91
 * Added mail-info and .plan-support by Brainiac / 28-5-91
 * Added domain info for Jedi's by Brainiac / 10-6-91
 * Converted KoBra-mud style to BSXmud by Orion / 23-4-92
 */

#define POST_DIR "room/post/postdir/"

string pre_title, title, al_title;      /* titles for short description */
int level;                              /* level of player */
int age;                                /* age of player */
int gender;                             /* gender of player */
int last_enter;                         /* time of last entry */
string called_from_ip;                  /* info for scheppers only */
string domain;                          /* schepper's domain */
int is_invis;                           /* do not give info about invis scheppers
                                           to players */
int last_read, last_arrived;            /* data about mail */

/*
 * NAME:        finger(string)
 *
 * ACTION:      give finger information about a player. scheppers get more info.
 */
finger(name)
{
    string line, total_output;
    object on;
    status schepper;
    int index;

    if(sscanf(name,"%s..%s",name,name)==2) return 1; /* fixed by Orion */
    name = lower_case(name);

    /* first find out if the player is logged on */
    on = find_player(name);
    if (on) {
        on->save_me(1); /* update info */
    }

    /* get the info */
    domain = 0; /* some old players might not have a domain var - Dworkin */
    if (!restore_object("users/" + name)) {
      if(file_size("/banish/"+name+".o")>=0) {
        write("That player has been banished.\n");
        return 1;
      }
        write("No such player.\n");
        return;
    }

    if (this_player()->query_level() >= 20) {
        schepper = 1;
        index = 56;     /* need more room for more info */
    } else {
        schepper = 0;
        index = 32;
    }

    /* player name and title */
    total_output = capitalize(name) + " " + title + " (" + al_title + ")\n";
    if (pre_title) {
        total_output = pre_title + " " + total_output;
    }

    if (schepper && domain)
    {
        if (!gender)
          total_output += "It ";
        else if (gender == 1)
          total_output += "He ";
        else
          total_output += "She ";
        if (level != 22)
          total_output += "is member of the domain "+capitalize(domain)+".\n";
        else
          total_output += "is Lord of the domain "+capitalize(domain)+".\n";
     }

    /* on during/last logon */
    if (is_invis && !schepper) {
        on = 0;                 /* no info about invis scheppers */
    }
    if (on) {
        line = "On for ";
    } else {
        line = "Last login: ";
    }
    if (stringp(last_enter)) {
        line += "some time";
    } else {
        line += single_time(time() - last_enter);
    }
    if (!on) {
        line += " ago";
    }
    if (schepper) {
        line = pad(line + " from " + called_from_ip, index);
    }

    if (on) {
        int idle_time;

        /* idle time info */
        if (query_ip_number(on)) {
            idle_time = query_idle(on);
            if (idle_time > 5 * 60) {
                line = pad(line, index) + idle_string(idle_time) + " idle time";

            }
        } else {
            line = pad(line, index) + "(link dead)";
        }
        total_output += line + "\n";

        if (schepper) {
            /* location info for scheppers */
            line = pad("Location: /" + file_name(environment(on)), index);
        } else if (level < 20) {
            /* age info for players, in hours */
            line = pad("Age: " + single_time(2 * age), index);
        } else {
            line = "";
        }
    } else if (!schepper) {
        /* no more info for players about persons not logged on */
        write(total_output + line + ".\n");
    }

    if (schepper || on)
    {
       /* level info */
       line += "Level: ";
       if (level < 20 || schepper) {
           line += level;
       } else {
           if (level == 20) {
               line += "Apprentice Schepper";
           } else if (level == 21) {
               line += "Junior Schepper";
           } else if (level == 22) {
               line += "Schepper";
           } else if (level == 23) {
               line += "Senior Schepper";
           } else if (level < 900) {
               line += "Arch Schepper";
           } else {
               line += "THE Schepper";
           }
       }
       total_output += line + "\n";

       if (schepper) {
           /* age info for scheppers */
           total_output += "Age: " + multiple_time(2 * age) + ".\n";
       }

       write(total_output);
    }

    /*
     * added mail-info and .plan-support
     */
    last_read = 0; last_arrived = 0;
    if (!restore_object(POST_DIR + name)
        || (last_read >= last_arrived))
    {
        write("No unread mail.\n");
    }
    else
    {
        write("New mail received " + ctime(last_arrived) + ";\n");
        if (!last_read)
            write("  never read mail.\n");  
        else
            write("  unread since " + ctime(last_read) + "\n");
    }

    if (level > 20)
    {
        if (file_size("/players/"+name+"/.plan") > 0)
        {
               write("Plan:\n");
           cat("/players/"+name+"/.plan");
        }
        else write("No plan.\n");
     }

}

/*
 * NAME:        pad(string, int)
 *
 * ACTION:      pad a string on a given length
 */
static pad(str, index)
{
    return str +
      extract("                                                        ",
              1, index - strlen(str));
}

/*
 * NAME:        single_time(int)
 *
 * ACTION:      Give the time as a string, in days, hours, minutes or seconds.
 */
static single_time(t)
{
    string str;

    if (t >= 86400) {
        t /= 86400;
        str = t + " day";
    } else if (t >= 3600) {
        t /= 3600;
        str = t + " hour";
    } else if (t >= 60) {
        t /= 60;
        str = t + " minute";
    } else {
        str = t + " second";
    }

    if (t == 0 || t > 1) str += "s";
    return str;
}

/*
 * NAME:        multiple_time(int)
 *
 * ACTION:      Give the time as a string, in days, hours, minutes and seconds.
 */
static multiple_time(t)
{
    string str;

    str = "";
    if (t >= 86400) {
        str += single_time(t) + " ";
        t = t % 86400;
    }
    if (t >= 3600) {
        str += single_time(t) + " ";
        t = t % 3600;
    }
    if (t >= 60) {
        str += single_time(t) + " ";
        t = t % 60;
    }
    return str + single_time(t);
}

/*
 * NAME:        idle_string(t)
 *
 * ACTION:      give the time in idle string format, "n days" or "12:34"
 */
static idle_string(t)
{
    string str;

    if (t >= 86400) {
        return single_time(t);
    }

    str = "" + t/36000; t = t % 36000;
    str += t/3600;      t = t % 3600;
    str += ":" + t/600; t = t % 600;
    str += t/60;

    return str;
}
