/*
 * A general purpose monster. Clone this object,
 * and call local functions to configure it.
 */

/*
 * If you are going to copy this file, in the purpose of changing
 * it a little to your own need, beware:
 *
 * First try one of the following:
 *
 * 1. Do clone_object(), and then configure it. This object is specially
 *    prepared for configuration.
 *
 * 2. If you still is not pleased with that, create a new empty
 *    object, and make an inheritance of this objet on the first line.
 *    This will automatically copy all variables and functions from the
 *    original object. Then, add the functions you want to change. The
 *    original function can still be accessed with '::' prepended on the name.
 *
 * The maintainer of this LPmud might become sad with you if you fail
 * to do any of the above. Ask other wizards if you are doubtful.
 *
 * The reason of this, is that the above saves a lot of memory.
 */

#include "creature.h"

/*
 * The heart beat is always started in living.h when we are attacked.
 */

string short_desc, long_desc, alias, alt_name, race;
int move_at_reset, aggressive;
object kill_ob;
status healing;         /* True if this monster is healing itself. */

string chat_head;       /* Vector with all chat strings. */
int chat_chance;

string a_chat_head;     /* Vector with all a_chat strings. */
int a_chat_chance;

object talk_ob;
string talk_func;       /* Vector of functions. */
string talk_match;      /* Vector of strings. */
string talk_type;       /* Vector of types. */
string the_text;
int have_text;


object dead_ob;
object init_ob;

int random_pick;
status soft_kill;

int spell_chance, spell_dam;
string spell_mess1, spell_mess2;
object me;
object create_room;

reset(arg)
{
    if (arg) {
        if (move_at_reset)
            random_move();
        return;
    }
    ::reset(0);
    is_npc = 1;
    enable_commands();
    add_commands();
    me = this_object();
    create_room = environment(me);
}

random_move()
{
    int n;
    n = random(4);
    if (n == 0)
        command("west");
    else if (n == 1)
        command("east");
    else if (n == 2)
        command("south");
    else if (n == 3)
        command("north");
}

short() {
    return short_desc;
}

long() {
    write (long_desc);
}

id(str) { return str == name || str == alias || str == race || str == alt_name; }

heart_beat()
{
    int c;

    age += 1;
    if(!test_if_any_here()) {
        if(have_text && talk_ob) {
            have_text = 0;
            test_match(the_text);
        } else {
            set_heart_beat(0);
            if (!healing)
                heal_slowly();
            return;
        }
    }
    if (kill_ob && present(kill_ob, environment(this_object()))) {
        if (random(2) == 1)
            return;             /* Delay attack some */
        attack_object(kill_ob);
        kill_ob = 0;
        return;
    }
    if (attacker_ob && present(attacker_ob, environment(this_object())) &&
      spell_chance > random(100)) {
        say(spell_mess1 + "\n", attacker_ob);
        tell_object(attacker_ob, spell_mess2 + "\n");
        call_other(attacker_ob, "hit_player", random(spell_dam));
    }
    attack();
    if (attacker_ob && whimpy && hit_point < max_hp/5 && brave < random(200))
        run_away();
    if(chat_chance || a_chat_chance){
        c = random(100);
        if(attacker_ob && a_chat_head) {
            if(c < a_chat_chance){
                c = random(sizeof(a_chat_head));
                tell_room(environment(), a_chat_head[c]);
            }
        } else {
            if(c < chat_chance && chat_chance){
                c = random(sizeof(chat_head));
                tell_room(environment(), chat_head[c]);
            }
        }
    }
    if(random_pick) {
        c = random(100);
        if(c < random_pick)
            pick_any_obj();
    }
    if(have_text && talk_ob) {
        have_text = 0;
        test_match(the_text);
    }
}

set_living(toggle)
{
    if (toggle) {
        enable_commands();
    } else {
        disable_commands();
    }
}

can_put_and_get(str)
{
    if (!str)
        return 0;
    return 1;
}

int busy_catch_tell;

catch_tell(str) {
    if (busy_catch_tell)
        return;
    busy_catch_tell = 1;
    if(talk_ob) {
        if(have_text) {
            test_match(the_text);
            the_text = str;
        } else {
            the_text = str;
            have_text = 1;
        }
    }
    busy_catch_tell = 0;
}

/*
 * Call the following functions to setup the monster.
 * Call them in the order they appear.
 */

set_name(n) {
    name = n;
    set_living_name(n);
    alignment = 0;              /* Neutral monster */
    cap_name = capitalize(n);
    short_desc = cap_name;
    long_desc = "You see nothing special.\n";
}

set_level(l) {
    if (level > 19) level = 19;
    level = l;
    weapon_class = level/2 + 3;
    armour_class = level/4;
    set_str(20 + (level * (3 + random(2))) + random(5));
    set_tough(20 + (level * (3 + random(2))) + random(5));
    set_dex(20 + (level * (3 + random(2))) + random(5));
    set_int(20 + (level * (3 + random(2))) + random(5));
    set_brave(20 + (level * (3 + random(2))) + random(5));
    set_social(20 + (level * (3 + random(2))) + random(5));
    hit_point = 50 + (level - 1) * 10;  /* Same as a player */
    max_hp = hit_point;
    spell_points = max_hp;
    /* This is for level 1 monsters. */
    if (experience == 0)
        experience = random(500);
}

/* Optional */
set_alias(a) { alias = a; }
/* Optional */
set_alt_name(a) { alt_name = a; }
/* Optional */
set_race(r) { race = r; }
/* optional */
set_hp(hp) { max_hp = hp; hit_point = hp; }
/* optional. Can only be lowered */
set_ep(ep) { if (ep < experience) experience = ep; }
/* optional */
set_al(al) { alignment = al; }
/* optional */
set_short(sh) { short_desc = sh; long_desc = short_desc + "\n";}
/* optional */
set_long(lo) { long_desc = lo; }
/* optional */
set_wc(wc) { weapon_class = wc; }
/* optional */
set_ac(ac) { armour_class = ac; }
/* optional */
set_move_at_reset() { move_at_reset = 1; }
/* optional
 * 0: Peaceful.
 * 1: Attack on sight (ONLY when alignment difference exist!)
 */
set_aggressive(a) {
    aggressive = a;
}
/*
 * optional
 * 0: monster kills normally.   
 * 1: monster kills softly -> player doesn't lose any stats.
 */
set_soft_kill(a)
{
    soft_kill = a;
}
/*
 * Now some functions for setting up spells !
 */
/*
 * The percent chance of casting a spell.
 */
set_chance(c) {
    spell_chance = c;
}
/* Message to the victim. */
set_spell_mess1(m) {
    spell_mess1 = m;
}
set_spell_mess2(m) {
    spell_mess2 = m;
}
set_spell_dam(d) {
    spell_dam = d;
}

/* Set the frog curse. */
set_frog() {
    frog = 1;
}

/* Set the whimpy mode */
set_whimpy() {
    whimpy = 1;
}

/*
 * Force the monster to do a command. The force_us() function isn't
 * always good, because it checks the level of the caller, and this function
 * can be called by a room.
 */
init_command(cmd) {
    command(cmd);
}

load_chat(chance, strs) {
    sizeof(strs);               /* Just ensure that it is an array. */
    chat_head = strs;
    chat_chance = chance;
}

/* Load attack chat */

load_a_chat(chance, strs) {
    sizeof(strs);               /* Just ensure that it is an array. */
    a_chat_head = strs;
    a_chat_chance = chance;
}

/* Catch the talk */

set_match(ob, func, type, match) {
    object old;

    if (sizeof(func) != sizeof(type) || sizeof(match) != sizeof(type))
        return;
    talk_ob = ob;
    talk_func = func;
    talk_type = type;
    talk_match = match;
}

set_dead_ob(ob)
{
    dead_ob = ob;
}

second_life()
{
    if(dead_ob)
        return call_other(dead_ob,"monster_died",this_object());
}

set_random_pick(r)
{
    random_pick = r;
}

pick_any_obj() {
    object ob;
    int weight;

    ob = first_inventory(environment(this_object()));
    while(ob) {
        if (call_other(ob, "get", 0) && call_other(ob, "short")) {
            weight = call_other(ob, "query_weight", 0);
            if (!add_weight(weight) || (random(dex) < 20)) {
                say(cap_name + " tries to take " + call_other(ob, "short", 0) +
                    " but fails.\n");
                return;
            }
            move_object(ob, this_object());
            say(cap_name + " takes " + call_other(ob, "short", 0) + ".\n");
            if (call_other(ob, "weapon_class", 0))
                call_other(ob, "wield", call_other(ob,"query_name"));
            else if (call_other(ob, "armour_class", 0))
                call_other(ob, "wear", call_other(ob,"query_name"));
            return;
        }
        ob = next_inventory(ob);
    }
}

set_init_ob(ob)
{
    init_ob = ob;
}

init() {

    create_room = environment(me);
    if(this_player() == me)
        return;
    if(init_ob)
        if(call_other(init_ob,"monster_init",this_object()))
            return;
    if (attacker_ob) {
        set_heart_beat(1); /* Turn on heart beat */
    }
    if(this_player() && !call_other(this_player(),"query_npc")) {
        set_heart_beat(1);
        if ((aggressive == 1 && (alignment * this_player()->query_alignment()
            < 0)) || (aggressive==1 && social<1))     /* unsocial attack all */
            kill_ob = this_player();
    }
}

query_create_room() { return create_room; }

query_race() { return race; }

query_soft_kill() {return soft_kill;}

test_match(str) {
    string who, str1, type, match, func;
    int i;

    while(i < sizeof(talk_match)) {
        if (talk_type[i])
            type = talk_type[i];
        match = talk_match[i];
        if (match == 0)
            match = "";
        if (talk_func[i])
            func = talk_func[i];
        if (sscanf(str,"%s " + type + match + " %s\n",who,str1) == 2 ||
           sscanf(str,"%s " + type + match + "\n",who) == 1 ||
           sscanf(str,"%s " + type + match + "%s\n",who,str1) == 2 ||
           sscanf(str,"%s " + type + " " + match + "\n",who) == 1 ||
           sscanf(str,"%s " + type + " " + match + " %s\n",who,str1) == 2)
        {
            return call_other(talk_ob, func, str);
        }
        i += 1;
    }
}

/*
 * The monster will heal itself slowly.
 */
heal_slowly() {
    hit_point += 120 / (INTERVAL_BETWEEN_HEALING * 2);
    if (hit_point > max_hp)
        hit_point = max_hp;
    spell_points += 120 / (INTERVAL_BETWEEN_HEALING * 2);
    if (spell_points > max_hp)
        spell_points = max_hp;
    healing = 1;
    if (hit_point < max_hp || spell_points < max_hp)
        call_out("heal_slowly", 120);
    else
        healing = 0;
}

/* Wield a weapon. */
wield(w) {
    if (name_of_weapon)
        stop_wielding();
    name_of_weapon = w;
    say(cap_name + " wields " + call_other(w, "query_name", 0) + ".\n");
}

/* Wear some armour. */
wear(a) {
    object old;

    if(head_armour) {
        old = call_other(head_armour, "test_type", call_other(a, "query_type"));
        if(old)
            return old;
        old = head_armour;
        call_other(a, "link", old);
    }
    head_armour = a;
    /* Calculate new ac */
    say(cap_name + " wears " + call_other(a, "query_name", 0) + ".\n");
    return 0;
}

stop_wielding() {
    if (!name_of_weapon) {
        /* This should not happen ! */
        log_file("wield_bug", "Weapon not wielded !\n");
        write("Bug ! The weapon was marked as wielded ! (fixed)\n");
        return;
    }
    call_other(name_of_weapon, "un_wield", dead);
    name_of_weapon = 0;
}

stop_wearing(name) {
    if(!head_armour) {
        /* This should not happen ! */
        log_file("wearing_bug", "armour not worn!\n");
        write("This is a bug, no head_armour\n");
        return;
    }
    head_armour = call_other(head_armour, "remove_link", name);
    if (!objectp(head_armour)) {
        head_armour = 0;
    }
    if (!is_npc)
        if(!dead)
            say(cap_name + " removes " + name + ".\n");
}
random_micro_move()
{
	    int n;
    n = random(4);
    if (n == 0)
	micro_w();
    else if (n == 1)
	micro_e();
    else if (n == 2)
	micro_s();
    else if (n == 3)
	micro_n();
}
