#include "/players/wing/std.h"

#define DRAWING_NAME 0
#define DRAWING_POLYGONS 1
mixed drawings;
static int current;

#define COLORS ({ "black", "blue", "green", "skyblue", "red", \
		  "pink", "brown", "lightgrey", "grey", "lightblue", \
		  "lightgreen", "cyan", "darkred", "mangenta", "yellow", \
		  "white" })

void reset( int flag )
{
  string a;
  
  if (flag)
    return;
  if (sscanf( FN, "%s#%s", a, a ) != 2)
    return;
  drawings = ({ });
  restore_object( "players/" + TP->query_real_name() + "/drawings" );
  current = -1;
}

void init()
{
  add_action( "help", "help" );
  add_action( "save", "save" );
  add_action( "list", "list" );
  add_action( "rename", "rename" );
  add_action( "check", "check" );
  add_action( "check2", "check2" );
  add_action( "colors", "colors" );
  add_action( "name", "name" );
  add_action( "work", "work" );
  add_action( "steal", "steal" );
  add_action( "remove", "remove" );
  add_action( "draw", "draw" );
  add_action( "change", "change" );
  add_action( "insert", "insert" );
  add_action( "erase", "erase" );
  add_action( "get_string", "string" );
  add_action( "create_something", "create" );
/*  add_action( "set", "set" );*/
}

#define SAVE_HELP	"save drawings\n"
#define LIST_HELP	"list drawings\n"
#define RENAME_HELP	"rename [drawing] [new name]\n"
#define CHECK_HELP	"check [drawing]\n"
#define CHECK2_HELP	"check2 [drawing]\n"
#define COLORS_HELP	"colors\n"
#define NAME_HELP	"name [new drawing]\n"
#define WORK_HELP	"work [drawing]\n"
#define STEAL_HELP	"steal [object] as [name]\n"
#define REMOVE_HELP	"remove [drawing]\n"
#define DRAW_HELP	"draw [color] [x0] [y0] ... [xn] [yn]\n"
#define CHANGE_HELP	"change [polygon] to [color]\n" + \
			"change [polygon] to [x0] [y0] ... [xn] [yn]\n"
#define INSERT_HELP	"insert [color] [x0] [y0] ... [xn] [yn] at [polygon]\n"
#define ERASE_HELP	"erase [polygon]\n"
#define STRING_HELP	"string [drawing]\n"
#define CREATE_OBJECT_HELP	"create object\n"
#define CREATE_ROOM_HELP	"create room\n"

#define ALL_HELP SAVE_HELP + LIST_HELP + RENAME_HELP +  CHECK_HELP + \
		 CHECK2_HELP + COLORS_HELP +  NAME_HELP +  WORK_HELP + \
		 STEAL_HELP + REMOVE_HELP + DRAW_HELP + CHANGE_HELP + \
		 INSERT_HELP + ERASE_HELP + STRING_HELP + \
		 CREATE_OBJECT_HELP + CREATE_ROOM_HELP


status help( string str )
{
  if (stringp( str ) && str == "pen" )
  {
    my_write( "With this magic pen you can do the following:\n" + ALL_HELP );
    return 1;
  }
  return 0;
}
	      
status save( string str )
{
  if (!stringp( str ) || lower_case( str ) != "drawings" )
    return 0;
  save_object( "players/" + TP->query_real_name() + "/drawings" );
}

status list( string str )
{
  int c;

  if (!stringp( str ) || lower_case( str ) != "drawings" )
    return 0;
  for (c=0; c < sizeof( drawings ); c++)
    my_write( (c == current ? "*" : "" ) + c + ": " +
	      drawings[c][ DRAWING_NAME ] );
  return 1;
}

int color_number( string color_name )
{
  int c;
  
  if (!stringp( color_name ))
    return -1;
  color_name = lower_case( color_name );
  for (c=0; c < 16; c++)
    if (COLORS[c] == color_name)
      return c;
  return -1;
}

string color_name( int color_number )
{
  if (color_number < 0 || color_number >= 16)
    return "invalid";
  return COLORS[ color_number ];
}

int find_drawing( string name )
{
  int c;

  for (c=0; c < sizeof( drawings ); c++)
    if (drawings[c][DRAWING_NAME] == name)
      return c;
  return -1;
}

status rename( string str )
{
  string new_name, old_name;
  int number, tmp;

  if (!stringp( str ))
    return 0;
  tmp = sscanf( str, "%d %s", number, new_name );
  if (tmp == 0 && sscanf( str, "%s %s", old_name, new_name ) == 2)
  {
    tmp = 2;
    number = find_drawing( old_name ); 
    if (number == -1)
    {
      my_notify_fail( "No drawing named " + old_name + "." );
      return 0;
    }
  }
  if (tmp == 2)
  {
    if (number >= sizeof( drawings ))
    {
      my_notify_fail( "There are not that many drawings." );
      return 0;
    }
    my_write( "Drawing " + drawings[number][DRAWING_NAME] + " renamed to "
	      + new_name + "." );
    drawings[number][DRAWING_NAME] = new_name;
    return 1;
  }
  else
  {
    my_notify_fail( "Usage: " + RENAME_HELP );
    return 0;
  }
}

status check( string str )
{
  int number, c, c1;
  string tmp;
  
  if (!stringp( str ) || str == "")
  {
    if (current != -1)
      number = current;
    else
    {
      my_notify_fail( "You are not working on any drawing." );
      return 0;
    }
  }
  else if (sscanf( str, "%d", number ) != 1)
  {
    number = find_drawing( str );
    if (number == -1)
    {
      my_notify_fail( "No drawing is named " + str + "." );
      return 0;
    }
  }
  if (number > sizeof( drawings ) - 1)
  {
    my_notify_fail( "There are not that many drawings." );
    return 0;
  }
  for (c=0; c < sizeof( drawings[number][DRAWING_POLYGONS] ); c++)
  {
    tmp = c + ": " + color_name( drawings[number][DRAWING_POLYGONS][c][0] ) +
      " (" + drawings[number][DRAWING_POLYGONS][c][1] + "," +
      drawings[number][DRAWING_POLYGONS][c][2] + ")";
    for (c1=3; c1 < sizeof( drawings[number][DRAWING_POLYGONS][c] ); c1+=2)
      tmp += "-(" + drawings[number][DRAWING_POLYGONS][c][c1] + "," +
	drawings[number][DRAWING_POLYGONS][c][c1+1] + ")";
    my_write( tmp );
  }
  current = number;
  return 1;
}

status check2( string str )
{
  int number, c, c1;
  string tmp;
  
  if (!stringp( str ) || str == "")
  {
    if (current != -1)
      number = current;
    else
    {
      my_notify_fail( "You are not working on any drawing." );
      return 0;
    }
  }
  else if (sscanf( str, "%d", number ) != 1)
  {
    number = find_drawing( str );
    if (number == -1)
    {
      my_notify_fail( "No drawing is named " + str + "." );
      return 0;
    }
  }
  if (number > sizeof( drawings ) - 1)
  {
    my_notify_fail( "There are not that many drawings." );
    return 0;
  }
  for (c=0; c < sizeof( drawings[number][DRAWING_POLYGONS] ); c++)
  {
    tmp = c + ": " + color_name( drawings[number][DRAWING_POLYGONS][c][0] ) +
      " " + drawings[number][DRAWING_POLYGONS][c][1] + " " +
      drawings[number][DRAWING_POLYGONS][c][2];
    for (c1=3; c1 < sizeof( drawings[number][DRAWING_POLYGONS][c] ); c1+=2)
      tmp += " " + drawings[number][DRAWING_POLYGONS][c][c1] + " " +
	drawings[number][DRAWING_POLYGONS][c][c1+1];
    my_write( tmp );
  }
  current = number;
  return 1;
}

status colors( string str )
{
  int c;
  
  for (c=0; c < 16; c++)
    my_write( c + ": " + COLORS[c] );
  return 1;
}

status name( string str )
{
  string tmp;
  
  if (!stringp( str ))
    return 0;
  drawings += ({ ({ str, ({ }) }) });
  current = sizeof( drawings ) - 1;
  my_write( "You decide name your new drawing " + str + "." );
  return 1;
}

status work( string str )
{
  int number;
  string tmp;
  
  if (!stringp( str ))
    return 0;
  if (sscanf( str, "%d %s", number, tmp ) == 1)
  {
    if (number < 0 || number >= sizeof( drawings ))
    {
      my_notify_fail( "There are not that many drawings." );
      return 0;
    }
  }
  else if( (number = find_drawing( str )) == -1)
  {
    my_notify_fail( "There is no drawing named " + str + "." );
    return 0;
  }
  my_write( "You descide to start working on the drawing " +
	    drawings[number][DRAWING_NAME] + "." );
  current = number;
  return 1;
}

#define CHECK_RANGE(a) \
if (c+a > sizeof( intstream )) \
{ \
  my_notify_fail( "Corrupt graphics data in object." );\
  return 0;\
}
		  
status steal( string str )
{
  mixed obj;
  string name;
  int *intstream, total_size, polygon_size, *polygon, c, c1, c2;
  mixed polygons;

  if (!stringp( str ) || sscanf( str, "%s as %s", obj, name ) != 2)
  {
    my_notify_fail( "Usage: " + STEAL_HELP );
    return 0;
  }
  obj = find_object( obj );
  if (obj == 0)
  {
    my_notify_fail( "Couldn't find that object." );
    return 0;
  }
  intstream = CONVHEX( obj->graph() );
  if (intstream == 0 || sizeof( intstream ) < 1)
  {
    my_notify_fail( "Corrupt graphics data in object." );
    return 0;
  }
  polygons = ({ });
  total_size = intstream[0];
  c=1;
  for (c1=0; c1 < total_size; c1++)
  {
    CHECK_RANGE(2);
    polygon_size = intstream[c++];
    polygon = ({ intstream[c++] });
    for (c2 = 0; c2 < polygon_size; c2++)
    {
      CHECK_RANGE(2);
      polygon += ({ intstream[c++], intstream[c++] });
    }
    polygons += ({ polygon });
  }
  if (c != sizeof( intstream ))
  {
    my_notify_fail( "Corrupt graphics data in object." );
    return 0;
  }
  drawings += ({ ({ name, polygons }) });
  current = sizeof( drawings ) - 1;
  return 1;
}

status remove( string str )
{
  string tmp;
  int number;
  
  if (!stringp( str ))
    return 0;
  if (sscanf( str, "%d %s", number, tmp ) != 1)
    if ((number = find_drawing( str )) == -1)
    {
      my_notify_fail( "There is no drawing named " + str + "." );
      return 0;
    }
  if (number >= sizeof( drawings ))
  {
    my_notify_fail( "There are not that many drawings." );
    return 0;
  }
  my_write( "You decide to remove the drawing " +
	    drawings[number][DRAWING_NAME] + "." );
  drawings = slice_array( drawings, 0, number - 1 ) +
    slice_array( drawings, number + 1, MYCKE );
  current = -1;
  return 1;
}    

status draw( string str )
{
  int x,y,tmp,color;
  int *polygon;
  string rest;
    
  if (!stringp( str ))
    return 0;
  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing." );
    return 0;
  }
  if (str == "")
  {
    my_notify_fail( "Usage: " + DRAW_HELP );
    return 0;
  }
  if (sscanf( str, "%d %s", color, rest ) == 2)
  {
    str = rest;
    if (color < 0 || color >= 16)
    {
      my_notify_fail( "The colornumber must be between 0 and 15." );
      return 0;
    }
  }
  else
  {
    if (sscanf( str, "%s %s", rest, str ) != 2)
    {
      my_notify_fail( "Usage: " + DRAW_HELP );
      return 0;
    }
    if ((color = color_number( rest )) == -1)
    {
      my_notify_fail( "There is no color named " + rest + "." );
      return 0;
    }
  }
  polygon = ({ color });
  while ((tmp = sscanf( str, "%d %d %s", x, y, str )) == 3)
    polygon += ({ x, y });
  if (tmp == 2)
    drawings[current][DRAWING_POLYGONS] += ({ polygon + ({ x, y }) });
  else if (tmp == 0)
    drawings[current][DRAWING_POLYGONS] += ({ polygon });
  else
  {
    my_notify_fail( "Usage: " + DRAW_HELP );
    return 0;
  }
  return 1;
}

status change( string str )
{
  int x,y,tmp,color,number;
  int *polygon;
  string rest;
    
  if (!stringp( str ))
    return 0;
  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing." );
    return 0;
  }
  if (str == "" || sscanf( str, "%d to %s", number, str ) != 2)
  {
    my_notify_fail( "Usage: change polygon to (color, colornumber or new " +
		    "coordinates." );
    return 0;
  }
  if (number < 0 || number > sizeof( drawings[current][DRAWING_POLYGONS] ))
  {
    my_notify_fail( "There are not that many plygons." );
    return 0;
  }
  if ((tmp = sscanf( str, "%d %d %s", x, y, rest )) == 0)
  {
    color = color_number( str );
    if (color == -1)
    {
      my_notify_fail( "There is no color named " + str + "." );
      return 0;
    }
  }
  else if (tmp == 1)
  {
    color = x;
    if (color < 0 || color >= 16)
    {
      my_notify_fail( "There are not that many colors." );
      return 0;
    }
  }
  else
  {
    polygon = ({ drawings[current][DRAWING_POLYGONS][number][0], x, y });
    str = rest;
    while ((tmp = sscanf( str, "%d %d %s", x, y, str )) == 3)
      polygon += ({ x, y });
    if (tmp == 2)
      drawings[current][DRAWING_POLYGONS][number] = polygon + ({ x, y });
    else if (tmp == 0)
      drawings[current][DRAWING_POLYGONS][number] = polygon;
    else
    {
      my_notify_fail( "Usage: " + CHANGE_HELP );
      return 0;
    }
    return 1;
  }
  drawings[current][DRAWING_POLYGONS][number][0] = color;
  return 1;
}

status insert( string str )
{
  string data;
  int color, x, y, *polygon, tmp, number;

  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing." );
    return 0;
  }
  if (!stringp( str ) || sscanf( str, "%s at %d", data, number ) != 2)
  {
    my_notify_fail( "Usage: " + INSERT_HELP );
    return 0;
  }
  if (sscanf( data, "%d %s", color, data ) == 2)
  {
    if (color < 0 || color >= 16)
    {
      my_notify_fail( "The colornumber must be between 0 and 15." );
      return 0;
    }
  }
  else
  {
    if (sscanf( data, "%s %s", str, data ) != 2)
    {
      my_notify_fail( "Usage: " + DRAW_HELP );
      return 0;
    }
    if ((color = color_number( str )) == -1)
    {
      my_notify_fail( "There is no color named " + str + "." );
      return 0;
    }
  }
  polygon = ({ color });
  while ((tmp = sscanf( data, "%d %d %s", x, y, data )) == 3)
    polygon += ({ x, y });
  if (tmp == 2)
    polygon += ({ x, y });
  else if (tmp != 0)
  {
    my_notify_fail( "Usage: " + INSERT_HELP );
    return 0;
  }
  drawings[current][DRAWING_POLYGONS] =
    slice_array( drawings[current][DRAWING_POLYGONS], 0, number - 1 ) +
    ({ polygon }) +
    slice_array( drawings[current][DRAWING_POLYGONS], number, MYCKE );
  return 1;
}

status erase( string str )
{
  int number;
    
  if (!stringp( str ))
    return 0;
  if (current == -1)
  {
    my_notify_fail( "You are not working on any drawing." );
    return 0;
  }
  if (str == "" || sscanf( str, "%d", number ) != 1)
  {
    my_notify_fail( "Usage: " + ERASE_HELP );
    return 0;
  }
  if (number < 0 || number > sizeof( drawings[current][DRAWING_POLYGONS] ))
  {
    my_notify_fail( "There are not that many plygons." );
    return 0;
  }
  drawings[current][DRAWING_POLYGONS] =
    slice_array( drawings[current][DRAWING_POLYGONS], 0, number - 1 ) +
    slice_array( drawings[current][DRAWING_POLYGONS], number + 1, MYCKE );
  return 1;
}

string create_polygon( int *polygon )
{
  string result;
  int c;

  result = SENDHEX( (sizeof( polygon ) - 1) / 2 ); /* size */
  result += SENDHEX( polygon[0] ); /* color */
  for (c=1; c < sizeof( polygon ); c += 2)
    result += SENDHEX( polygon[c] ) + SENDHEX( polygon[c+1] );
  return result;
}

string create_graph( mixed drawing )
{
  string result;
  int c;

  result = SENDHEX( sizeof( drawing ) ); /* Number of polygons */
  for (c=0; c < sizeof( drawing ); c++)
    result += create_polygon( (int *)drawing[c] );
  return result;
}

status get_string( string str )
{
  int number, c, c1;
  string tmp;
  
  if (!stringp( str ) || str == "")
  {
    if (current != -1)
      number = current;
    else
    {
      my_notify_fail( "You are not working on any drawing." );
      return 0;
    }
  }
  else if (sscanf( str, "%d", number ) != 1)
  {
    number = find_drawing( str );
    if (number == -1)
    {
      my_notify_fail( "No drawing is named " + str + "." );
      return 0;
    }
  }
  if (number > sizeof( drawings ) - 1)
  {
    my_notify_fail( "There are not that many drawings." );
    return 0;
  }
  my_write( create_graph( drawings[number][DRAWING_POLYGONS] ) );
  current = number;
  return 1;
}

status create_object()
{
  object obj;
  
  obj = clone_object( "/players/wing/generic_object" );
  obj->set_graph( create_graph( drawings[current][DRAWING_POLYGONS] ) );
  move_object( obj, TP );
  return 1;
}

status create_room()
{
  object room;

  room = clone_object( "/players/wing/generic_room" );
  room->set_graph( create_graph( drawings[current][DRAWING_POLYGONS] ) );
  TP->move_player( "disappears into his creation", room );
  return 1;
}

status create_something( string str )
{
  if (!stringp( str ))
    return 0;
  str = lower_case( str );
  if (str == "object")
    return create_object();
  else if (str == "room")
    return create_room();
  return 0;
}

string short()
{
  return "a pen";
}

string long()
{
  return "A pen that can be used to create objects and rooms.\n";
}

status id( string str )
{
  return stringp( str ) && lower_case( str ) == "pen";
}

status get()
{
  return 1;
}

status drop()
{
  return 1;
}
