/*
 *      File    :       player.c
 *      Part of :       BSX-system for graphical MUDs
 *      by      :       Bram Stolk
 *      date    :       13/09/91
 *      lastmod :       07/05/92
 */

/*
	Orion changed everything he could lay his hands on
	Date: unknown
*/

/*
 * Object-oriented-hierarchy :
 *      a player is a creature is a living is an xthing
 *
 */
# include "creature.h"
# include "/system/ascii.h"

#define WIZ                     0
#define ARCH                    1
#define USERDIR			"users/"

/*
 * player variables - These are saved at end of session for next session
 */

string title;                   /* Our official title */
string password;                /* This players crypted password. */
string al_title;                /* Indicate how 'noble' a player is */
string called_from_ip;          /* IP number that was used last time */
string quests;                  /* A list of all quests */
string last_ed;                 /* last file edited */
static string sued;
int    last_enter;              /* time of last session */
int    tot_value;               /* Saved values of this player. */
string current_path;            /* Current directory */
string access_list;             /* extra dirs this player has access to */
string country;                 /* player's country of origin */
string planet;                  /* player's planet of origin */
string pre_title;               /* pre-title for schepper's */
string email_addr;		/* Address on internet */

/*
 * Static variables - These are used by player.c, but not saved in player.o
 */

static status VALID_OK;         /* need this in kludge for teleporting */
static status link_dead;        /* is this player linkdead? */
static int time_to_save;        /* Time to autosave. */
static string password2;        /* Temporary when setting new password */
static string saved_where;
static object other_copy;
static string client_version;

static string xserverlogfile;	/* Filename used when opening a local client */
static int xserverlogrow;
/*
 * Some code :-)
 */

static logon() {
  time_to_save = 500;
  cat("/WELCOME");
  write("What is your name: ");
  input_to("logon2");
  call_out("time_out", 120);
  return 1;
}

static end_session() {
  tell_object(this_object(),"@"+"TMS");
  if(sued) {
    restore_object("users/"+sued);
    sued=0;
    load_auto_obj(auto_load);
    move_object(clone_object("obj/soul"),this_object());
    return 1;
  } else {
  destruct(this_object());
  }
}

static try_throw_out(str) {
  object ob;
  if (str == "" || (str[0] != 'y' && str[0] != 'Y')) {
    write("Welcome another time then !\n");
    end_session();
    return;
  }
  ob = first_inventory(other_copy);
  while(ob) {
    object next_ob;
    next_ob = next_inventory(ob);
    /*
     * Don't move the soul.
     */
    if (!call_other(ob, "id", "soul")) {
      move_object(ob, this_player());
      ob->set_wielded_by(this_player());
      ob->set_worn_by(this_player());
    }
    ob = next_ob;
  }
  name_of_weapon = other_copy->query_wielded();
  head_armour = other_copy->query_head_armour();
  ob = environment(other_copy);
  call_other(other_copy, "quit",1);
  write("Points restored from the other object.\n");
  move_player_to_start(ob);
  log_file("ENTER", " (throw)\n");
}

static invalid_enter() {
 return 0;
}

static logon2(str) {
  if (!str || str == "") {
    end_session();
    return;
  }
  if (name != "logon") {
    illegal_patch("logon2");
    end_session();
    return;
  }
  str = lower_case(str);
  if (!valid_name(str)) {
    input_to("logon2");
    write("Give name again: ");
    return;
  }
  if (file_size("/banish/" + str + ".o") >= 0) {
    write("That name is reserved.\n");
    end_session();
    return;
  }
  if (str == "guest")
  {
    logon_guest(); /* Wing 921104 */
    return;
  }
  if (!restore_object(USERDIR + str)) {
    write("New character.\n");
  }
  money = 0;  /* extra security, character is reloaded later */
  /*
   * Don't do this before the restore !
   */
  name = str;                 /* Must be here for a new player. */
  time_to_save = age + 500;
  dead = ghost;
  if (is_invis)
    cap_name = "Someone";
  else
    cap_name = capitalize(name);
  write("Password: ");
  if (name == "guest")
    write("(just CR) ");
  if (level != -1)
    input_to("check_password", 1);
  else
    input_to("new_password", 1);
  attacker_ob = 0;
  alt_attacker_ob = 0;
  return;
}

/* Called by command 'save' */
save_character() {
    save_me(1);
    return 1;
}

reset(arg) {
  if (myself) return;
  ::reset(arg);
  get_unique_idstr();
/*
 *   With arg = 0 this function should only be entered once!
 */
  if (creator(this_object())) {
    illegal_patch("Cloned player.c");
    
    return;
  }
  name = "logon";
  cap_name = "Logon";
  title = "the title less";
  al_title = "neutral";
  gender = -1;  /* force to change it */
  sued=0;
}

short() {
  string str;

  if (link_dead)
    str = "frozen";
  else
    str = al_title;
  if (is_invis)
    return 0;
  if (ghost)
    return "ghost of " + cap_name;
  if (frog)
    return cap_name + " the frog";
  if (!pre_title)
    return cap_name + " " + title + " (" + str + ")";
  else return pre_title + " " + cap_name + " " + title + " (" + str + ")";
}

long() {
  write(short());
  if (level < 20) write(" is a virtual reality.\n");
  if (level == 20) write(" is an apprentice Schepper.\n");
  if (level == 21) write(" is a Schepper.\n");
  if (level == 22) write(" is an Experienced Schepper.\n");
  if (level == 23) write(" is a Senior Schepper.\n");
  if (level == 24) write(" is a Mega Schepper.\n");
  if (level > 24 && level < 1000) write(" is a Giga Schepper.\n");
  if (level >= 1000) write(" is THE Schepper.\n");
  if (country)
    write(capitalize(query_pronoun())+" comes from "+country+".\n");
  if (ghost || frog)
    return;
  write(query_shape()+"\n");
}

static su(str) {
  if (level > 24) {
    if(sued) {
      write("You already su'ed to "+name+".\n");
      return 1;
    }
    if(file_size("/users/"+str+".o")<1) {
      write("Player does not exist.\n");
      return 1;
    }
    sued=name;
    save_me(1);
    restore_object("users/"+str);
    move_object(clone_object("obj/soul"),this_object());
    write("You are now "+capitalize(name)+"\n");
    return 1;
  }
}
query_sued() {
	return sued;
}
score() {
  if (ghost) {
    write("You are in an immaterial state with no scores.\n");
    return 1;
  }
  write(short()+" (level "+level+", "+query_gender_string()+")\n");
  write("You have earned "+experience+" experience points, and "+money+" credits.\n");
  write("Hitpoints: "+hit_point+" ("+max_hp+")  Spellpoints: "+spell_points+" ("+2*max_hp+")\n");
  show_age();
  write("\n");
  return 1;
}

/* Identify ourself. */
id(str, lvl) {
  /*
   *  Some wizzies make invisibility items usuable by
   *  players , and this will prevent cheating.
   */
  if(level < 20)
    if(str == name || str == "ghost of " + name)
      return 1;
  /*
   *  I think this looks stupid. When I am invisible it is
   *  because I want to work in PEACE.
   */
  if (is_invis && lvl <= level)
    return 0;
  if (ghost)
    return str == "ghost of " + name;
  if (str == name)
    return 1;
  return 0;
}

query_title() {
  return title;
}

set_level(lev) {
  if (lev > 21 || lev < level && level >= 20)
    return illegal_patch("set_level");
  if(this_player()) {
   if(lev>20) {
    if(!(query_ip_number(this_player())&&(this_player()->query_level()>23))) {
       return illegal_patch("set_level");
    }
   }
  }
  ::set_level(lev);
  if (level == 20) {
    tell_object(myself, "Adding Schepper commands...\n");
    wiz_commands();
  }
  if (level == 21) {
    tell_object(myself, "Adding more Schepper commands...\n");
    wiz_commands2();
  }
}

set_title(t) {
  if (!t) {
    write("Your title is " + title + ".\n");
    return 1;
  }
  title = t;
  trunc_title();
  return 1;
}

static pretitle(str) {
  if (!str) {
    write("Your pretitle has been cleared.\n");
    pre_title = 0;
    return 1;
  }
  write("Your pretitle is now: " + str + "\n");
  pre_title = str;
  trunc_title();
  return 1;
}

trunc_title() {
  int len;

  len = strlen(name) + 4;
  if (pre_title) {
    if (strlen(pre_title) + 1 + len > 80)
      pre_title = 0;
    else
      len += 1 + strlen(pre_title);
  }
  if (len + strlen(title) > 80)
    title = "";
  len += strlen(title);
  if (len + strlen(al_title) > 80)
    al_title = "";
}

static wiz_commands2() {
  if (this_object() != this_player())
    return;
  add_action("makedir", "mkdir");
  add_action("su","su");
  add_action("removedir", "rmdir");
  add_action("pwd", "pwd");
  add_action("more", "more");
  add_action("echo_to", "echoto");
  add_action("echo", "echo");
  add_action("echo_all", "echoall");
  add_action("home", "home");
  add_action("remove_file", "rm");
  add_action("list_files", "ls");
  add_action("cat_file", "cat");
  add_action("edit", "ed");
  add_action("cp", "cp");
  add_action("mv", "mv");
  add_action("log", "log");
  add_action("grep", "grep");
  add_action("clone", "clone");
  add_action("destruct_local_object", "dest");
  add_action("load", "load");
  add_action("reset_obj", "reset");
  add_action("tail_file", "tail");
  add_action("cd", "cd");
}

static wiz_commands() {
  if (this_object() != this_player())
    return;
  add_action("local_commands", "localcmd");
  add_action("wiz_score_list", "schepperlist");
  add_action("force_player", "force");
  add_action("spell_zap", "zap");
  add_action("stat", "stat");
  add_action("heal", "heal");
  add_action("update_object", "update");
  add_action("set_title", "title");
  add_action("pretitle", "pretitle");
  add_action("teleportt", "goto");
  add_action("in_room", "in");
  add_action("at_liv", "at");
  add_action("list_peoples", "people");
  add_action("setmmin", "setmmin");
  add_action("setmmout", "setmmout");
  add_action("setmin", "setmin");
  add_action("setmout", "setmout");
  add_action("review", "review");
  add_action("shut_down_game", "shutdown");
  add_action("summon", "trans");
  add_action("invis", "invis");
  add_action("vis", "vis");
  add_action("find", "where");
  add_action("tell_all_scheppers","tas");
}

player_commands() {
  add_action("levels","levels");
  add_action("score", "score");
  add_action("emote", "emote");
  add_action("emote", ":");
  add_action("find", "find");
  add_action("finger", "finger");
  add_action("specify_country", "country");
  add_action("specify_email", "email");
  add_action("save_character", "save");
  add_action("quit", "quit");
  add_action("shout_to_all", "shout");
  add_action("inventory", "i");
  add_action("look", "look");
  add_action("look", "l");
  add_action("examine", "examine");
  add_action("examine", "exa", 1);
  add_action("help", "help");
  add_action("index", "index");
  add_action("change_password", "password");
  add_action("idea", "idea");
  add_action("typo", "typo");
  add_action("bug", "bug");
  add_action("communicate", "'", 1);
  add_action("converse", "converse");
  add_action("spell_breeze","breeze");
  add_action("spell_wind","wind");
  add_action("spell_storm","storm");
  add_action("spell_frost","frost");
  add_action("spell_fire","fire");
  add_action("spell_maim","maim");
  add_action("spell_plague","plague");
  add_action("spell_AIDS","aids");
  add_action("spell_nuke","nuke");
  add_action("spell_armageddon","armageddon");
  add_action("spell_bang","bang");
  add_action("toggle_whimpy", "wimpy");
  add_action("stop_hunting_mode", "stop");
  add_action("pose", "pose");
  add_action("who", "who");
  add_action("serve_rqs","#RQS");
  add_action("serve_rqo","#RQO");
  add_action("serve_ver","#VER");
  add_action("micro_n","micro_n");
  add_action("micro_s","micro_s");
  add_action("micro_e","micro_e");
  add_action("micro_w","micro_w");
  add_action("take_a_new_look","take_a_new_look");
}

/* No killing around here :) 
kill(str) {
  if (ghost)
    return 0;
  return ::kill(str);
}
*/

static log_shout(str) {
  str=check_message(str);
  shout(str + "\n");
  return 1;
}

static echo_all(str) {
  if (!str) {
    write("Echoall what?\n");
    return 1;
  }
  log_shout(str);
  write("You echo: " + str + "\n");
  return 1;
}

static echo(str) {
  if (!str) {
    write ("Echo what?\n");
    return 1;
  }
  say(str+"\n");
  write("You echo: "+str+"\n");
  return 1;
}

static echo_to(str) {
  object ob;
  string who;
  string msg;
  if (!str || sscanf(str, "%s %s", who, msg) != 2) {
    write("Echoto what ?\n");
    return 1;
  }
  ob = find_living(lower_case(who));
  if (!ob) {
    write("No player with that name.\n");
    return 1;
  }
  tell_object(ob,msg+"\n");
  write("You echo: "+msg+"\n");
  return 1;
}

quit(arg) {
  save_me(0);
  drop_all(1);
  if (!is_invis && !link_dead) {
    say(cap_name+" left the realms of bsx, and now has to deal with the harsh cold world outside.\n@RMO"+idstr+".@RFS");
  }
  while (first_inventory(myself)) {
    destruct(first_inventory(myself));
  }
  write("@"+"PRO");
  end_session();
  return 1;
}

/* multiple line 'say' done by Orion 23-4-1992 */
communicate(str) {
  string verb, dummy, temp;

  verb = query_verb();
  if (verb != "say" && verb != "'") {
    verb = extract(verb, 1);
    if (str)
      {
      str = verb + " " + str;
      }
    else
      str = verb;
  } else if (!str) {
    write("Say what ?\n");
    verb=cap_name;
    if(ghost) verb=short();
    say(verb+" stutters.\n");
    return 1;
  }
  verb=cap_name;
  if(ghost) verb=short();
  str=check_message(str);
  while(strlen(str+verb)>71) {
    temp="";
    dummy="";
    while(strlen(temp+dummy+verb)<72) {
      if(temp=="") {
        temp=dummy;
      } else {
        temp+=" "+dummy;
      }
      dummy="";
      if(sscanf(str,"%s %s",dummy,str)!=2) {
         dummy=str;
         str="";
      }
    }
  if(sscanf(str,"%s %s",dummy,str)!=2) {
    say(verb+" says: "+str+"\n");
    write("You say: "+str+"\n");
    str="";
  }
  if(temp!="") {
    say(verb+" says: "+temp+"\n");
    write("You say: "+temp+"\n");
  }
    if(strlen(dummy)>70) {
      say(verb+" says: "+dummy+"\n");
      write("You say: "+dummy+"\n");
      dummy="";
    }
    if(dummy!="") str=dummy+" "+str;
  }
  if(str!="") {
  say(verb+" says: "+str+"\n");
  write("You say: "+str+"\n");
  }
  return 1;
}

static heart_beat() {
  if (ghost)
    return;
  ::heart_beat();
  if (age > time_to_save) {
    save_me(1);
    time_to_save = age + 500;
  }
  if (!query_ip_number(this_object())) {
    link_dead = 1;
    call_out("dest_link_dead", 2400);
    if (!is_invis)
      say(cap_name+" is suddenly frozen in time!\n");
    set_heart_beat(0);
  } else {
    if (link_dead) {
      remove_call_out("dest_link_dead");
      link_dead = 0;
    }
  }
}

dest_link_dead() {
  if (!is_invis)
    say(cap_name + " is destructed by anti-linkdead matter.\n");
  quit(1);
}

check_point() {
  save_me(1);
}

/*
 * Update our aligment.
 */

add_alignment(a) {
  if (!intp(a)) {
    write("Bad type argument to add_alignment.\n");
    return;
  }
  alignment = alignment*9/10 + a;
  if (level > 20)
    return;
  if (alignment > KILL_NEUTRAL_ALIGNMENT * 100) {
    al_title = "saint";
    return;
  }
  if (alignment > KILL_NEUTRAL_ALIGNMENT * 20) {
    al_title = "pure of heart";
    return;
  }
  if (alignment > KILL_NEUTRAL_ALIGNMENT * 4) {
    al_title = "decent";
    return;
  }
  if (alignment > - KILL_NEUTRAL_ALIGNMENT * 4) {
    al_title = "neutral";
    return;
  }
  if (alignment > - KILL_NEUTRAL_ALIGNMENT * 20) {
    al_title = "tempted by the Dark side";
    return;
  }
  if (alignment > - KILL_NEUTRAL_ALIGNMENT * 100) {
    al_title = "defected to the Dark side";
    return;
  }
  al_title = "avoid at all cost!";
}

set_alignment(al) {
  al_title = al;
  trunc_title();
}

pick_up(str) {
  if (ghost) {
    write("You fail.\n");
    return 1;
  }
  return ::pick_up(str);
}

static pick_item(obj) {
  if (ghost) {
    write("You fail.\n");
    return 1;
  }
  return ::pick_item(obj);
}

query_weight() { return 80; }

/*
 * Temporary move the player to another destination and execute
 * a command.
 */

static in_room(str) {
  object room;
  object old_room;
  string cmd;
  if (!str)
    return 0;
  if (sscanf(str, "%s %s", room, cmd) != 2) {
    write("Usage: in ROOM CMD\n");
    return 1;
  }
  room = valid_read(room, ARCH);
  if (!room) {
    write("Invalid file name.\n");
    return 1;
  }
  old_room = environment();
  move_object(myself, room);
  command(cmd);
  move_object(myself, old_room);
  return 1;
}

static at_liv(str) {
  object liv;
  object old_loc;
  string comm, creature;

  if (!str) {
    write("USAGE: at <living> <command>\n");
    return 1;
  }
  if (sscanf(str, "%s %s", creature, comm) != 2) {
    write("USAGE: at <living> <command>\n");
    return 1;
  }
  liv = find_living(creature);
  if (!liv) {
    write("Living not found!\n");
    return 1;
  }
  old_loc = environment(this_player());
  move_object(this_player(), environment(liv));
  command(comm);
  move_object(this_player(), old_loc);
  return 1;
}

shout_to_all(str) {
  if (spell_points < 0) {
    write("The Force is not strong enough.\n");
    return 1;
  }
  if (level < 20)
    spell_points -= 20;
  if (!str) {
    write("Shout what ?\n");
    return 1;
  }
  if (ghost) {
    write("You fail.\n");
    return 1;
  }
  log_shout(cap_name + " shouts: " + str);
  write("Ok.\n");
  return 1;
}

emote(str) {
  if (!str) {
    write("emote what ?\n");
    return 1;
  }
  write(cap_name + " " + str + "\n");
  say(cap_name + " " + str + "\n");
  return 1;
}

specify_country(str) {
  if (!str) {
    write("Your country-info has been cleared.\n");
    country = 0;
    return 1;
  }
  write(cap_name + " comes from " + str + "\n");
  country = str;
  return 1;
}

specify_email(str) {
  if (!str) {
    write("Your email-info has been cleared.\n");
    email_addr = 0;
    return 1;
  }
  email_addr = str;
  write("email address set to "+email_addr+"\n");
  return 1;
}

query_email_addr() {
  return email_addr;
}

load_me() {
  if((this_player()->query_level()<24)||(!query_ip_number(this_player())))
    return illegal_patch("load_me");
  if(file_size("/users/"+name+".o")<1) {
    write("load_me failed: no .o file.\n");
    return 1;
  }
  restore_object("users/"+name);
  write("load_me succeeded.\n");
  tell_object(this_object(),"You feel different all of a sudden.\n");
  return 1;
}

inventory() {
  object ob;
  if (test_dark())
    return 1;
  ob = first_inventory(myself);
  write("");
  while(ob) {
    string str;
    str = call_other(ob, "short", 0);
    if (str) {
      write(str + "\n");
      it = str;
    }
    ob = next_inventory(ob);
  }
  return 1;
}

examine(str) {
  return look("at " + str);
}

look(str) {
  object ob, ob_tmp;
  string item, tmp;
  int max;
  if (!str) {
    tmp = call_other(environment(), "long", 0);
    if (tmp)
      write( tmp ); /* Wing 921104 */
    return 1;
  } 
  if (sscanf(str, "at %s", item) == 1 || sscanf(str, "in %s", item) == 1) {
    item = lower_case(item);
    ob = present(item, this_player());
    if (!ob && call_other(environment(this_player()), "id", item))
      ob = environment(this_player());
    if (!ob)
      ob = present(item, environment(this_player()));
    if (!ob) {
      write("There is no "+item+" here.\n");
      return 1;
    }
    it = item;
    tmp = call_other(ob, "long", item);
    if (tmp)
      write( tmp ); /* Wing 921104 */
    if (!call_other(ob, "can_put_and_get", item))
      return 1;
    if (living(ob)) {
      object special;
      special = first_inventory(ob);
      while(special) {
        string extra_str;
        extra_str = call_other(special, "extra_look");
        if (extra_str)
          write(extra_str + ".\n");
        special = next_inventory(special);
      }
    }
    ob_tmp = first_inventory(ob);
      while(ob_tmp && call_other(ob_tmp, "short") == 0)
        ob_tmp = next_inventory(ob_tmp);
      if (ob_tmp) {
        if (living(ob)) {
          write("     " + capitalize(item) + " is carrying:\n");
        } else
          write("     " + capitalize(item) + " contains:\n");
      }
      max = MAX_LIST;
      ob = first_inventory(ob);
      while(ob && max > 0) {
        string sh;
        sh = call_other(ob, "short", 0);
        if (sh)
          write(sh + "\n");
        ob = next_inventory(ob);
        max -= 1;
      }
    return 1;
  }
  write("Look AT something, or what ?\n");
  return 1;
}

static check_password(p) {
  if (password == 0)
    write("You have no password ! Set it with the 'password' cmd.\n");
  else if ((name != "guest") && (crypt(p, password) != password)) {
    write("Wrong password!\n");
    end_session();
    return;
  }
  write("\n");
  check_gender();
  log_file("ENTER", cap_name + " entered at " + ctime(time()) + " with " + experience + " exp. ("+query_ip_number(myself)+")\n");
}

/*
 * Give a new password to a player.
 */
static new_password(p) {
  if (!p || p == "") {
    write("Try again another time then.\n");
    end_session();
    return;
  }
  if (strlen(p) < 6) {
    write("The password must have at least 6 characters.\n");
    write("Password: ");
    input_to("new_password", 1);
    return;
  }
  if (password == 0) {
    password = p;
    write("\nPassword: (again) ");
    input_to("new_password", 1);
    return;
  }
  if (password != p) {
    write("You changed !\n");
    end_session();
    return;
  }
  password = crypt(password, 0);      /* Generate new seed. */
  email_addr = 0;
  gender = -1;
  planet = 0;
  level = 0;
  call_other("system/levels","advance",this_object());
  strength = 20;
  tough = 20;
  dex = 20;
  intel = 20;
  brave = 20;
  social = 20;
  max_hp = 20;
  hit_point = max_hp;
  write("\n");
  check_gender();
  log_file("NEWPLAYER", cap_name + ".\n");
}

static check_gender() {
  if (gender == -1) {
    write("What is your gender, Male, Female or Other?\n");
    input_to("get_gender",0);
    return;
  } else
    check_race();
}

static move_player_to_start(where) {
  saved_where = where;
  move_player_to_start2(where);
} /* move */
static get_gender(gender_string) {
  gender_string = lower_case(gender_string);
  if (gender_string[0] == 'm') {
    write("Welcome, Sir!\n");
    set_male();
  }
  else if (gender_string[0] == 'f') {
    write("Welcome, Madam!\n");
    set_female();
  }
  else if (gender_string[0] == 'o') {
    write("Welcome, Creature!\n");
    set_neuter();
  }
  else {
    write("Sorry, but that is too weird for this game!\n");
    write("Are you, male, female or other (type m, f or o): ");
    input_to("getgender");
    return;
  }
  check_race();
}

static check_race() {
  if (!planet) {
    write("What planet are you from?\n");
    cat("/SPECIES");
    write("Please choose one of the numbers above: ");
    input_to("get_race",0);
    return;
  } else
    move_player_to_start2(saved_where);
}

static get_race(race) {
  int r;
  if ((sscanf(race,"%d\n",r) != 1) || (r<1) || (r>7)) {
    write("Sorry.... choose 1..7\nWhat planet are you from? : ");
    input_to("get_race");
    return;
  }
  graphic = call_other("obj/appearance", "get_appearance", r);
  planet  = call_other("obj/appearance", "get_planet", r);
  al_title= call_other("obj/appearance", "get_inhabitant", r);
  move_player_to_start2(0);
}

static logon_guest() /* Wing 921104 */
{
  remove_call_out("time_out");
  enable_commands();
  add_action("serve_rqs","#RQS");
  add_action("serve_rqo","#RQO");
  add_action("serve_ver","#VER");
  add_action("just_terminate","quit");
  write(
"@DFSintro.0C050904FB0405FB05FBFB04FB040209F6090AF50AF5F605030FF10F0FF00FF0F1F0F1040414EB1415EB15EBEB040619E6E5E6E51A191A04001FE1E0E1E01F1F1F060E2ABB2A5A5F5A5F904F904FBB04003F7B3F6B4A6B4A7B04003FAB3F9B449B44AB0C0E6ABB6A858F858F706A706A5A9F5A9F957A957AAB9FAB9FBB040EAA5AB55ADABBD0BB040EAABBB5BBDA5AD05A@SCEintro.@RFS"
  );
  cat("/NEWS");
  client_version="NONE    ";
  write("@RQV");
  write("\nChecking for client, please wait...");
  call_out( "logon_guest2", 2, 0 );
}

logon_guest2( int times )
{
  if (client_version == "NONE    ")
    if (times < 5)
    {
      write( ".." );
      call_out( "logon_guest2", 2, times + 1 );
      return;
    }
    else
    {
      write( "\nYou don't seem to be running any client. I can start a " +
	     "client locally\nand open a window on your X-server, if you " +
	     "are running X-windows, that is.\n" +
	     "Do you want me to try that?\n" );
      input_to( "start_clientp" );
    }
  else
    logon_guest3();
}

logon_guest3()
{
  int r;
  
  name = "guest";
  cap_name = "Guest";
  title = "the visitor";
  email_addr = 0;
  gender = 0;
  level = 1;
  strength = 20;
  tough = 20;
  dex = 20;
  intel = 20;
  brave = 20;
  social = 20;
  max_hp = 20;
  hit_point = max_hp;
  r = random(7)+1;
  graphic = call_other("obj/appearance", "get_appearance", r);
  planet  = call_other("obj/appearance", "get_planet", r);
  al_title= call_other("obj/appearance", "get_inhabitant", r);
  time_to_save = age + 500;
  last_ed = 0;
  local_weight = 0;
  move_object(clone_object("obj/soul"),this_object());
  set_living_name(name);
  set_heart_beat(1);
  add_commands();
  player_commands();
  move_object(myself, "room/crossroads");
  say("Another fortunate creature visits the realms of BSX, Please welcome Guest.\n");
  say(query_desc()+"@RFS");
  link_dead = 0;
  write("");
  cat("/GUEST");
  called_from_ip = query_ip_name();
  last_enter = time();
  glance();
  current_path = "players/" + name;
}

start_clientp( str )
{
  if (!stringp( str ) ||
      lower_case( str ) != "yes" && lower_case( str ) != "y")
    logon_guest3();
  else
  {
    write( "You'll have to give me permisson to open windows on your " +
	   "X-server, by typing\n\"xhost + robin.lysator.liu.se\" in a " +
	   "shell-window. Then you have to tell me\nwhere to open the " +
	   "window (probably \"" +
	   query_ip_name() + ":0\"),\nor just type CR to cancel.\n" );
    input_to( "which_xserver" );
  }
}

which_xserver( xserver )
{
  if (!stringp( xserver ) || xserver == "")
  {
    logon_guest3();
    return;
  }
  write( "Trying to open a window on \"" + xserver + "\".\n" +
	 "Hopefully a window will be opened on your screen, and you can " +
	 "log on normally.\nIf something goes wrong error-messages will " +
	 "be displayed here.\n" );
  xserverlogfile = "/obj/master"->load_client( xserver );
  xserverlogrow = 0;
  call_out( "check_client_progress", 2, 0 );
}

check_client_progress( int times )
{
  string tmp;

  if (times >= 30)
  {
    destruct( this_object() );
    return;
  }
  while (tmp = read_file( xserverlogfile, xserverlogrow, 1))
  {
    xserverlogrow++;
    if (tmp == "END\n")
    {
      write( "Seems like there was some errors, should I try again?\n" );
      input_to( "start_clientp" );
      return;
    }
    else
      write( tmp );
  }
  call_out( "check_client_progress", 2, times + 1 );
}

static just_terminate()
{
  destruct( this_object() );
}

static move_player_to_start2(where) {
  object ob;
  string tmp_name;
  /*
   * See if we are already playing.
   * We must blank our own name, or we could find ourselves !
   */
  tmp_name = name;
  name = 0;

  other_copy = find_player(tmp_name);
  name = tmp_name;
  enable_commands();
  if (other_copy) {
    write("You are already playing !\n");
    write("Throw the other copy out ?\n");
    input_to("try_throw_out");
    return;
  }
  restore_object(USERDIR+name);     /* get most recent data */
  time_to_save = age + 500;
  last_ed = 0;
  local_weight = 0;
  ob = first_inventory(this_object());
  while (ob) {
    local_weight += ob->query_weight();
    ob = next_inventory(ob);
  }
  if (!head_armour)
    armour_class = 0;
  if (!name_of_weapon)
    weapon_class = WEAPON_CLASS_OF_HANDS;
  /*
   * Now we can enter the game. Check tot_value if the game
   * crashed, and the values of the player was saved.
   */
  move_object(clone_object("obj/soul"),this_object());
  if (tot_value) {
    write("You find " + tot_value + " credits of your lost money!\n");
    money += tot_value;
    tot_value = 0;
  }
  remove_call_out("time_out");
  set_living_name(name);
  set_heart_beat(1);
  add_commands();
  player_commands();
  if (level >= 20)
    wiz_commands();
  if (level >= 21)
    wiz_commands2();
  write(
"@DFSintro.0C050904FB0405FB05FBFB04FB040209F6090AF50AF5F605030FF10F0FF00FF0F1F0F1040414EB1415EB15EBEB040619E6E5E6E51A191A04001FE1E0E1E01F1F1F060E2ABB2A5A5F5A5F904F904FBB04003F7B3F6B4A6B4A7B04003FAB3F9B449B44AB0C0E6ABB6A858F858F706A706A5A9F5A9F957A957AAB9FAB9FBB040EAA5AB55ADABBD0BB040EAABBB5BBDA5AD05A@SCEintro.@RFS"
  );
  cat("/NEWS");
  client_version="NONE    ";
  write("@RQV");
  if (where)
    move_object(myself, where);
  else {
    move_object(myself, "room/crossroads");
    }
    load_auto_obj(auto_load);
  if (!is_invis) {
    if (link_dead)
      say(cap_name+" thaws back to life again!\n");
    else
      say("Another fortunate creature enters the realms of BSX, Please welcome "+cap_name+".\n");
    say(query_desc()+"@RFS");
  } else {
    if (level < 24)
      write("YOU ARE INVISIBLE !\n\n");
  }
  link_dead = 0;
  write("");
  if (name == "guest")
    cat("/GUEST");
  if (level >= 20)
    cat("/WIZNEWS");
  if (query_ip_name() != called_from_ip && called_from_ip)
    write("Your last login was from " + called_from_ip + ". ");
  called_from_ip = query_ip_name();
  if (stringp(last_enter)) last_enter = 0;
  write("Time last login: "+ctime(last_enter)+" local time.\n");
  last_enter = time();
  write(call_other("room/post/p_master","query_mail",name));
  glance();
  current_path = "players/" + name;
}

static list_files(path) {
  string path_two, switcher;

  if (!path)
    path = "/" + current_path;
  if (sscanf(path, "-%s %s", switcher, path_two) != 2)
    if (sscanf(path, "-%s", switcher) == 1)
       path_two = "/" + current_path;
  if (switcher)
    ls(path_two + "/ " /* , switcher */);
  else
    ls(path + "/ ");
  return 1;
}

index() {
    write("");
    ls("/doc/helpdir/ ");
    write("\n");
    return 1;
}

tail_file(str) {
  string path;
  int chunks;

  path = 0; chunks = 0;
  if (!str)
    return 0;
  if (sscanf(str, "%d %s", chunks, path) != 2) {
    if (sscanf(str, "%d", chunks) == 1) {
      write("Syntax: tail [number_of_chunks] <path>\n");
      return 1;
    } else {
      if (sscanf(str, "%s", path) != 1) {
         write("Syntax: tail [number_of_chunks] <path>\n");
         return 1;
      }
    }
  }
  if (chunks) {
    if (!tail(chunks /* , path */))
      return 0;
  } else {
    if (!tail(path))
      return 0;
  }
  return 1;
}

cat_file(path) {
  if (!path)
    return 0;
  if (!cat(path))
    write("No such file.\n");
  return 1;
}

help(what) {
  if (what == "tools" && level >= 20) {
    cat("/doc/wiz_help");
    return 1;
  }
  if (what) {
    cat("/doc/helpdir/" + what);
    return 1;
  }
  cat("/doc/help");
  return 1;
}

tell(str) {
  if (ghost) {
    write("You fail.\n");
    return 1;
  }
  if (spell_points < 0) {
    write("@The Force is not strong enough.\n");
    return 1;
  }
  return ::tell(str);
}

list_peoples() {
  object list;
  int    i, a, p, num;
  string name, s, t;

  list = users();
  write("There are now " + sizeof(list) + " players");
  for (i = 0, a = 0; i < sizeof(list); i++)
    if (query_idle(list[i]) >= 5 * 60) a++;
  if (a) write(" (" + (sizeof(list) - a) + " active)");
  write(". " + query_load_average() + "\n");
  for (i = 0; i < sizeof(list); i++) {
    name = list[i]->query_real_name();
    if (!name) name = list[i]->query_name();
    if (!name) name = "logon";
    name = capitalize(name);
    if (list[i]->short() == 0) name = "(" + name + ")";
    s = "" + list[i]->query_level();
    s = extract("                ", strlen(s) + strlen(name)) + s;
    write(query_ip_sname(list[i]) + "\t" + name + s + "  ");
    a = list[i]->query_age();
    if (a / 43200 > 9)
      write(a / 43200 + "D");
    else if (a / 43200 > 0)
      write(a / 43200 + " D");
    else if (a / 1800 > 9)
      write(a / 1800 + "h");
    else if (a / 1800 > 0)
      write(a / 1800 + " h");
    else if (a / 30 > 9)
      write(a / 30 + "m");
    else
      write(a / 30 + " m");
    write(" ");
    if (query_idle(list[i]) >= 5 * 60) write("I"); else write(" ");
    if (list[i]->query_sued()) write("SU"); else write("  ");
    s = "";
    s = extract(capitalize(s), 0, 5);
    write(s + extract("        ", strlen(s)));

    if (environment(list[i])) {
      s = file_name(environment(list[i]));
      if (sscanf(s, "players/%s", t) == 1) s = "~" + t;
      write(s);
    }
    write("\n");
  }
  return 1;
}

query_ip_sname(pl) {
	string lname,sname;
	int i;
	lname=query_ip_name(pl);
	if(strlen(lname)<19) {
		sname=lname;
	} else {
		sname="";
		for(i=0;i<19;i++) sname=sname+ascii(lname[i]);
	}
	for(strlen(sname);strlen(sname)<20;sname=sname+" ") {}
	return sname;
}
tell_all_scheppers(str) {
  int i;
  object list;
  if (!str) {
    write("Syntax: tas <str>\n");
    return 1;
  }
  list = users();
  for(i = 0; i<sizeof(list); i++) {
    object who;
    who = list[i];
    if(!who->query_busy() && who->query_level()>19) {
      object soul;
      soul = present("soul",who);
      if(soul && !soul->query_silence())
      tell_object(who,this_player()->query_name()+
                      " Tells All Scheppers: "+str+"\n");
    }
  }
  return 1;
}

static update_object(str) {
  object ob;
  if (!str) {
    write("Update what object ?\n");
    return 1;
  }
  str = valid_read(str, ARCH);
  if (!str) {
    write("Invalid file name.\n");
    return 1;
  }
  ob = find_object(str);
  if (!ob) {
    write("No such object.\n");
    return 1;
  }
  destruct(ob);
  write(str + " will be reloaded at next reference.\n");
  return 1;
}

static check_dest(source, dest) {
  string temp;

  if (file_size(dest) == -2) {
  /* Directory destination */

    temp = explode(source, "/");
    if (dest[strlen(dest) - 1] != '/') dest += "/";
    return (dest + temp[sizeof(temp) - 1]);
  }
  return dest;
}

static cp(str) {
  string source, dest;

  if (!str) return;
  if (sscanf(str, "%s %s", source, dest) != 2) {
    write("Syntax: cp <source> <destination>\n");
    return 1;
  }
  if (!valid_read(source)) {
    write("Invalid source-file.\n");
    return 1;
  }
  dest = check_dest(source, dest);
  if (!valid_write(dest)) {
    write("Invalid destination-file.\n");
    return 1;
  }
  if (!copy_file(source, dest))
    write("No such file.\n");
  else
    write("Ok.\n");
  return 1;
}

static mv(str) {
  string source, dest;

  if (!str) return;
  if (sscanf(str, "%s %s", source, dest) != 2) {
    write("Syntax: mv <source> <destination>\n");
    return 1;
  }
  if (!valid_write(source)) {
    write("Invalid source-file.\n");
    return 1;
  }
  dest = check_dest(source, dest);
  if (!valid_write(dest)) {
    write("Invalid destination-file.\n");
    return 1;
  }
  if (!rename_file(source, dest))
    write("No such file.\n");
  else
    write("Ok.\n");
  return 1;
}

static edit(file) {
  if (!file) {
    if (!last_ed) {
      write("Edit what file ?\n");
      return 1;
    }
    write("Re-editing: "+last_ed+"\n");
    file = last_ed;
  }
  last_ed = file;
  file = valid_write(file);
  if ((!file)&&(level<1000)) {
    write("You can only edit your own files.\n");
    last_ed = 0;
    return 1;
  }
  ed(file);
  return 1;
}

static grep(str) {
  string expr, files, c1,c2;
  string f_array;
  int i, a;

  if (!str || sscanf(str, "%s %s", expr, files) != 2) {
    write("Usage: grep <string> file[,file,file,...]\n");
    return 1;
  }
  if (expr == "" || strlen(expr) < 2 || expr[0] == ' ') {
    write("Illegal search-string.\n");
    return 1;
  }
  if (sscanf(files, "%s,%s", c1, c2) == 2)
    f_array = explode(files, ",");
  else
    f_array = ({files});
  for (i = 0; i < sizeof(f_array); i++) {
    a = 0;
    c1 = f_array[i];
    while(c1[a] == ' ') a++;
    f_array[i] = extract(c1, a, strlen(c1) - 1);
  }
  grep(expr, f_array);
  return 1;
}

static log() {
  cat("/log/"+name);
  return 1;
}

static heal(name) {
  object ob;

  if (!name)
    return 0;
  it = lower_case(name);
  ob = find_living(it);
  if (!ob) {
    write("No such person is playing now.\n");
    return 1;
  }
  call_other(ob, "heal_self", 100000);
  tell_object(ob, "You are healed by "+cap_name+".\n");
  write("Ok.\n");
  return 1;
}

static stat(name) {
  object ob;

  if (!name)
    return 0;
  it = lower_case(name);
  ob = present(name, environment());
  if (!ob || !living(ob))
    ob = find_living(it);
  if (!ob) {
    write("No such person is playing now.\n");
    return 1;
  }
  call_other(ob, "show_stats", 0);
  return 1;
}

static shut_down_game(str) {
  if (!str) {
    write("You must give a shutdown reason as argument.\n");
    return 1;
  }
  shout("Game is shut down by " + capitalize(name) + ".\n");
  log_file("GAME_LOG", ctime(time()) + " Game shutdown by " + name +
             "(" + str + ")\n");
  shutdown();
  return 1;
}

/*
 * Check that a player name is valid. Only allow
 * lowercase letters.
 */
valid_name(str) {
  int i, length;
  if (str == "logon") {
    write("Invalid name");
    return 0;
  }
  length = strlen(str);
  if (length > 11) {
    write("Too long name.\n");
    return 0;
  }
  i=0;
  while(i<length) {
    if (str[i] < 'a' || str[i] > 'z') {
      write("Invalid characters in name:" + str + "\n");
      write("Character number was " + (i+1) + ".\n");
      return 0;
    }
    i += 1;
  }
  return 1;
}

/*
 * This one is called when the player wants to change his password.
 */
static change_password(str) {
  if (password != 0 && !str) {
    write("Give old password as an argument.\n");
    return 1;
  }
  if (password != 0 && password != crypt(str, password)) {
    write("Wrong old password.\n");
    return 1;
  }
  write("New password: ");
  password2 = 0;
  input_to("change_password2", 1);
  return 1;
}

static change_password2(str) {
  if (!str) {
    write("Password not changed.\n");
    return;
  }
  if (password2 == 0) {
    password2 = str;
    write("Again: ");
    input_to("change_password2", 1);
    return;
  }
  if (password2 != str) {
    write("Wrong! Password not changed.\n");
    return;
  }
  password = crypt(password2, 0);     /* Generate new seed */
  password2 = 0;
  write("Password changed.\n");
}

static bug(str) {
  if (!str) {
    write("Give an argument.\n");
    return 1;
  }
  log_file("BUGS", "\n");
  log_file("BUGS", cap_name + " (" +
         file_name(environment(this_object())) + "):\n");
  log_file("BUGS", str + "\n\n");
  smart_report("Bug " + cap_name + "\n" + str);
  write("Ok.\n");
  return 1;
}

static typo(str) {
  if (!str) {
    write("Give an argument.\n");
    return 1;
  }
  log_file("TYPO", cap_name + " (" +
       file_name(environment(this_object())) + "):\n");
  log_file("TYPO", str + "\n\n");
  smart_report("Typo " + cap_name + "\n" + str);
  write("Ok.\n");
  return 1;
}

static idea(str) {
  if (!str) {
    write("Give an argument.\n");
    return 1;
  }
  log_file("IDEA", cap_name + ":\n");
  log_file("IDEA", str + "\n\n");
  smart_report("Idea " + cap_name + "\n" + str);
  write("Ok.\n");
  return 1;
}

static converse() {
  write("Give '**' to stop.\n");
  write("]");
  input_to("converse_more");
  return 1;
}

static converse_more(str) {
  string cmd;
  if (str == "**") {
    write("Ok.\n");
    return;
  }
  if (str[0] == '!') {
    sscanf(str, "!%s", cmd);
    command(cmd);
  } else if (str != "") {
    ::communicate(str);
  }
  write("]");
  input_to("converse_more");
}


static toggle_whimpy() {
  whimpy = !whimpy;
  if (whimpy)
    write("Wimpy mode.\n");
  else
    write("Brave mode.\n");
  return 1;
}

add_exp(e) {
  if (this_player() && this_player() != this_object() &&
      query_ip_number(this_player()) && level < 20 && e >= 25000)
    log_file("ADD_EXP", ctime(time()) + " " + name + "(" + level + 
             ") " + e + " exp by " + this_player()->query_real_name() + 
             "(" + this_player()->query_level() + ")" +"\n");
  ::add_exp(e);
  if (level <= 19)
    add_worth(e);
}

add_money(m) {
  if (level < 20 && m >= 10000) {
    log_file("ADD_MONEY", extract(ctime(time()), 4, 18) + " " 
             + name + "(" + level + "), "+m+" cr. got");
    if (query_ip_number() && this_player() != this_object()) {
      log_file("ADD_MONEY", " by " + this_player()->query_real_name() +
            " (" + this_player()->query_level() + ")\n");
    } else
      if (this_player() != previous_object()) {
            log_file("ADD_MONEY", " by " + file_name(previous_object()) + "\n");
      } else {
         log_file("ADD_MONEY", "\n");
      }
  }
  ::add_money(m);
  return 1;
}

static drop_all_money(silent) {
  if (money >= 10000) {
    log_file("ADD_MONEY", extract(ctime(time()), 4, 18) + " " + 
        file_name(environment()) + " " + name + "(" + level + ") "+
        money+" cr. (d)\n");
  }
  return ::drop_all_money(silent);
}

second_life() {
  if (level >= 20)
    return illegal_patch("second_life");
  ghost = 1;
  set_idstr("obj/ghost");
  if ((!attacker_ob || !attacker_ob->query_soft_kill()) && level > 1) {
    level = level - 1;
    max_hp -= 8;
  }
  msgin = "drifts around";
  msgout = "blows";
  headache = 0;
  intoxicated = 0;
  stoned = 0;
  stomach = 0;
  hunter = 0;
  hunted = 0;
  attacker_ob = 0;
  alt_attacker_ob = 0;
  tell_object(myself, "\nYou die.\nYou have a strange feeling.\n" +
              "You can see your own dead body from above.\n\n");
  if(environment(this_object())) call_other(environment(this_object()),"resend_state");
  return 1;
}

remove_ghost() {
    if (!ghost)
        return 0;
    write("You feel a very strong force.\n");
    write("You are sucked away...\n");
    write("You reappear in a more solid form.\n");
    say("Some mist disappears.\n");
    say(cap_name + " appears in a solid form.\n");
    ghost = 0;
    set_idstr(file_name(this_object()));
    if(environment(this_object())) call_other(environment(this_object()),"resend_state");
    dead = 0;
    msgin = "arrives";
    msgout = "leaves";
    /*
    * code to correct level :-)
    */
    save_me(1);
    return 1;
}

stop_hunting_mode() {
  if (!hunted) {
    write("You are not hunting anyone.\n");
    return 1;
  }
  call_other(hunted, "stop_hunter");
  hunted = 0;
  write("Ok.\n");
  return 1;
}

no_spell() {
  write("You are too young, and not intelligent enough for that spell.\n");
  return 1;
}

spell_breeze(str) {
  object ob;
  if(test_dark()) return 1;
  if((level * intel) < 0 ) return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(100)>50) {
    write("You failed to cast a spell at your opponent.\n");
    if (spell_points > 4)
      spell_points -= 5;
    else spell_points = 0;
       return 1;
  }
  breeze_object(ob);
  return 1;
}

spell_wind(str) {
  object ob;
  if (test_dark())
    return 1;
  if (level*intel < 40)
    return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(100)>50) {
    write("You failed to beam your opponent.\n");
    if (spell_points > 9)
      spell_points -= 10;
    else
      spell_points = 0;
    return 1;
  }
  wind_object(ob);
  return 1;
}

spell_storm(str) {
  object ob;
  if (test_dark())
    return 1;
  if (level*intel < 90)
    return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(10)>5) {
    write("You failed to beam your opponent.\n");
    if (spell_points > 14) spell_points -= 15;
      else spell_points = 0;
     return 1;
  }
  storm_object(ob);
  return 1;
}

spell_frost(str) {
  object ob;
  if (test_dark())
    return 1;
  if (level*intel < 120)
    return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(10)>5) {
    write("You failed to beam your opponent.\n");
    if (spell_points > 19) spell_points -= 20;
      else spell_points = 0;
     return 1;
  }
  frost_object(ob);
  return 1;
}

spell_fire(str) {
  object ob;
  if (test_dark())
    return 1;
  if (level*intel < 160)
    return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(10)>5) {
    write("You failed to beam your opponent.\n");
    if (spell_points > 24) spell_points -= 25;
      else spell_points = 0;
     return 1;
  }
  fire_object(ob);
  return 1;
}

spell_maim(str) {
  object ob;
  if (test_dark())
    return 1;
  if (level*intel < 250)
    return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(10)>5) {
    write("You failed to beam your opponent.\n");
    if (spell_points > 29) spell_points -= 30;
      else spell_points = 0;
     return 1;
  }
  maim_object(ob);
  return 1;
}

spell_plague(str) {
  object ob;
  if (test_dark())
    return 1;
  if (level*intel < 360)
    return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(10)>5) {
    write("You failed to beam your opponent.\n");
    if (spell_points > 34) spell_points -= 35;
      else spell_points = 0;
     return 1;
  }
  plague_object(ob);
  return 1;
}

spell_AIDS(str) {
  object ob;
  if (test_dark())
    return 1;
  if (level*intel < 490)
    return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(10)>5) {
    write("You failed to beam your opponent.\n");
    if (spell_points > 39) spell_points -= 40;
      else spell_points = 0;
     return 1;
  }
  AIDS_object(ob);
  return 1;
}

spell_nuke(str) {
  object ob;
  if (test_dark())
    return 1;
  if (level*intel < 600)
    return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(10)>5) {
    write("You failed to beam your opponent.\n");
    if (spell_points > 44) spell_points -= 45;
      else spell_points = 0;
     return 1;
  }
  nuke_object(ob);
  return 1;
}

spell_armageddon(str) {
  object ob;
  if (test_dark())
    return 1;
  if (level*intel < 740)
    return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(10)>5) {
    write("You failed to beam your opponent.\n");
    if (spell_points > 49) spell_points -= 50;
      else spell_points = 0;
     return 1;
  }
  armageddon_object(ob);
  return 1;
}

spell_bang(str) {
  object ob;
  if (test_dark())
    return 1;
  if (level*intel < 880)
    return no_spell();
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  if (ob == myself) {
    write("Sadomasochism is prohibited by the Pope.\n");
    return 1;
  }
  if (random(10)>5) {
    write("You failed to beam your opponent.\n");
    if (spell_points > 54) spell_points -= 55;
      else spell_points = 0;
     return 1;
  }
  bang_object(ob);
  return 1;
}

static spell_zap(str) {
  object ob;
  if (!str)
    ob = attacker_ob;
  else
    ob = present(lower_case(str), environment(this_player()));
  if (!ob || !living(ob)) {
    write("At whom ?\n");
    return 1;
  }
  zap_object(ob);
  return 1;
}

static force_player(str) {
  string who, what;
  object ob;
  if (!str)
    return 0;
  if (sscanf(str, "%s to %s", who, what) == 2 ||
      sscanf(str, "%s %s", who, what) == 2) {
    ob = find_living(who);
    if (!ob) {
      write("No such player.\n");
      return 1;
    }
    if (level <= call_other(ob, "query_level")) {
      write("You may not force an equal or more powerful Schepper!\n");
      tell_object(ob,cap_name+" tried to force you: "+what+"\n");
      return 1;
    }
    tell_object(ob,cap_name+" forces you to: "+what+"\n");
    command(what, ob);
    write("Ok.\n");
    return 1;
  }
  return 0;
}

static clone(str) {
  object ob;
  if (!str) {
    write("Clone what object ?\n");
    return 1;
  }
  str = valid_read(str, ARCH);
  if (!str) {
    write("Invalid file.\n");
    return 1;
  }
  ob = clone_object(str);
  say(cap_name + " fetches something from another dimension.\n");
  move_object(ob, environment());
  if (call_other(ob, "get"))
    transfer(ob, myself);
  write("Ok.\n");
  return 1;
}

pose() {
  if (level >= 4) {
    write("You send a Super Nova into the sky.\n");
    say(cap_name + " uses the Force.\n");
    log_shout("A gigantic Super Nova explodes in the sky.");
    return 1;
  }
  return 0;
}

static destruct_local_object(str) {
  object ob;
  if (!str) {
    write("Destruct what ?\n");
    return 1;
  }
  if (str == "all") {
    dest_inventory();
    return 1;
  }
  str = lower_case(str);
  ob = present(str, this_player());
  if (!ob)
    ob = present(str, environment(this_player()));
  if (!ob) {
    write("No " + str + " here.\n");
    return 1;
  }
  if (environment(ob) != this_object()) {
    say("@RMO"+call_other(ob, "query_idstr")+".@RFS");
    write("@RMO"+call_other(ob, "query_idstr")+".@RFS");
  }
  say(call_other(ob, "short")+" is desintegrated by "+cap_name+".\n");
  destruct(ob);
  write("Ok.\n");
  return 1;
}

dest_inventory() {
  object ob, next_ob;
  string str;
  int nd;

  ob = first_inventory(myself);
  write("");
  while (ob) {
    next_ob = next_inventory(ob);
    str = call_other(ob, "short", 0);
    nd = call_other(ob, "id", "ND");
    if (!nd) {
      write("Destruct: " + str + " : Ok.\n");
      destruct(ob);
    }
    ob = next_ob;
  }
  write("\n");
  return 1;
}

static load(str) {
  object env;
  if (!str) {
    write("Load what ?\n");
    return 1;
  }
  str = valid_read(str, ARCH);
  if (!str) {
    write("Invalid file name.\n");
    return 1;
  }
  env = environment();
  /* We just call a non existing function in the object, and that way
   * force it to be loaded.
   */
  call_other(str, "???");
  write("Ok.\n");
  return 1;
}

static reset_obj(str) {
  object ob;
  if (!str) {
    write("Reset what?\n");
    return 1;
  }
  str = valid_read(str, ARCH);
  if (!str) {
     write("Invalid path!\n");
     return 1;
  }
  ob = find_object(str);
  if (!ob) {
    write("Object not found.\n");
    return 1;
  }
  call_other(ob, "reset", 1);
  write("Ok.\n");
  return 1;
}

static home() {
  move_player("home#players/" + name + "/workroom");
  get_all(environment());
  return 1;
}

static advance(path, owner) {
  string player, tail;
  object p;
  int i, n, sz;

  if (path[0] == '~') {
    if (path == "~") {
       return "players/" + name;
    } else if (sscanf(path, "~%s/%s", player, tail) == 2) {
      if (player == "") {
        player = name;
      }
      path = "players/" + player + "/" + tail;
    } else {
      return "players/" + extract(path, 1);
    }
  } else
    if (path[0] == '/') {
      path = extract(path, 1);
    } else
      if (owner) {
        path = "players/" + owner + "/" + path;
      } else {
        path = current_path + "/" + path;
      }
      p = explode("/" + path, "/");
      if (!p) {
        return ".";
      }
      sz = sizeof(p);
      n = 0;
      for (i = 0; i < sz; i += 1) {
        if (p[i] != "" && p[i] != ".") {
          if (p[i] == "..") {
            if (n == 0) {
              return 0;
            }
            n -= 1;
          } else {
            p[n] = p[i];
            n += 1;
          }
        }
      }
      if (n == 0) {
        return ".";
      }
      if (p[n - 1] == " ") {      /* ls */
        p[n - 1] = ".";
      }
      while (n < sz) {
        p[n] = 0;
        n += 1;
      }
      return implode(p, "/");
}

static check_access_list(str) {
  object list;
  int i;
  string dummy;

  list = explode(access_list, "#");
  for (i = 0; i < sizeof(list); i++) {
    if (sscanf(str, list[i] + "/%s", dummy) == 1)
      return 1;
  }
  return 0;
}

valid_write(str) {
  string who, file, owner;

  owner = name;
  if (previous_object() &&
    sscanf(file_name(previous_object()), "players/%s/", who) == 1) {
    if (who == "" || who == ".")
      return 0;
    owner = who;
  }
  str = advance(str, who);
  if (!str) {
    return 0;
  }
  if (sscanf(str, "obj/%s", file) == 1)

  if (sscanf(str, "obj/%s", file) == 1)
    if (file == "living.h" || file == "living.c" || file == "living.i" ||
        file == "creature.h" || file == "creature.c" || file == "creature.i" ||
        file == "xthing.c" || file == "player.c" || file == "player.i" ||
        file == "master.c")
      if (level>123)
	return str;
      else
        return 0;
  if (sscanf(str, "log/admin/%s", file) == 1 ||
    sscanf(str, "log/SNOOP%s", file) == 1) return 0;
  if (sscanf(str, "players/%s/%s", who, file) == 2)
    if (who == owner || level > 23)
      return str;
  if (sscanf(str, "log/%s", who) == 1) 
    if (who == owner || level > 23)
      return str;
  if (sscanf(str, "log/%s.%s", who, file) == 2) 
    if (who == owner || level > 23)
      return str;
  if (sscanf(str, "open/%s", file) == 1)
    return str;
  if (sscanf(str, "ftp/%s", file) == 1)
    return str;
  if (level < 24) return 0;
  if (sscanf(str, "doc/%s", file) == 1)
    return str;
  if (sscanf(str, "obj/%s", file) == 1)
    return str;
  if (sscanf(str, "room/%s", file) == 1)
    return str;
  if (sscanf(str, "players/%s", file) == 1)
    if (sscanf(str, "%s/%s", file, who) > 0 || capitalize(str) == str)
      return str;
  if (sscanf(str, "users/%s", file) == 1)
	{
		if(level<25) return 0;
		return str;
	}
  if (level < 24) return 0;
    return str;
}

valid_read(str, lvl) {
  string who, file;

  if (VALID_OK) {
    /* horrible kludge to fix teleport */
    lvl = ARCH;
    VALID_OK = 0;
  } else if (lvl == WIZ) {
    file = valid_write(str);
    if (file)
      return file;
  }
  str = advance(str);
  if (!str) {
    return 0;
  }
/*  if (sscanf(str, "players/%s", file) == 1 && level < 23 
      && lvl == WIZ && sscanf(file, "%s/.plan", who) != 1 &&
      (sscanf(file, "%s/%s/", who, file) != 2 || file != "open" ||
      who != capitalize(who))) {
    return 0;
  } */
  if ((sscanf(str, "log/admin/%s", file) == 1 && level < 23) ||
      sscanf(str, "log/admin/SNOOP%s", file) == 1) return 0;
    return str;
}

crea_ext(str)   /* create extension */ {
  int i;
  i = strlen(str) - 1;
  if (str[i-1] == '.' && str[i] == 'c') return str;
  return str + ".c";
}

static wiz_score_list(arg) {
  if (arg)
    wizlist(arg);
  else
    wizlist();
  return 1;
}

static remove_file(str) {
  if (!str)
    return 0;
  rm(str);
  return 1;
}

static local_commands() {
  localcmd();
  return 1;
}

/*
 * Recursively compute the values of the inventory.
 */
compute_values(ob) {
  int v;
  while(ob) {
    int tmp;
    tmp = ob->query_value();
    if (tmp > 1250)
      tmp = 1250;
    v += tmp;
    if (first_inventory(ob))
      v += compute_values(first_inventory(ob));
    ob = next_inventory(ob);
  }
  return v;
}

save_me(value_items) {
  if (value_items)
    tot_value = compute_values(first_inventory(this_object()));
  else
    tot_value = 0;
  compute_auto_str();
  save_object(USERDIR + name);
}

illegal_patch(what) {
  write("KAZAAAAM! You just got struck by a bolt from the Interior of the Game.\n");
  log_file("ILLEGAL",
           call_other(this_player(), "query_real_name") + " " +
           what + "\n");
  return 0;
}

load_auto_obj(str) {
  string file, argument, rest;
  object ob;

  while(str && str != "") {
    if (sscanf(str, "%s:%s^!%s", file, argument, rest) != 3) {
      write("Auto load string corrupt.\n");
      return;
    }
    str = rest;
    ob = find_object(file);
    if (!ob)
      continue;
    ob = clone_object(file);
    if (argument)
      call_other(ob, "init_arg", argument);
    move_object(ob, this_object());
  }
}

compute_auto_str() {
  object ob;
  string str, dummy;

  auto_load = "";
  ob = first_inventory(this_object());
  while(ob) {
    str = call_other(ob, "query_auto_load");
    ob = next_inventory(ob);
    if (!str || sscanf(str, "%s^!", dummy))
      continue;
    auto_load = str + "^!" + auto_load;
  }
}

smart_report(str) {
  string who;
  string current_room;

  current_room = file_name(environment(this_object()));
  if (sscanf(current_room, "players/%s/", who) != 1)
    return;
  log_file(who + ".rep", current_room + " " + str + "\n");
}

query_quests(str) {
  string tmp, rest, rest_tmp;
  int i;

  if (str == 0) {
   return  quests;
  }
  rest = quests;
  while(rest) {
    if (str == rest)
      return 1;
    i = sscanf(rest, "%s#%s", tmp, rest_tmp);
    if (i == 0)
      return 0;
    if (tmp == str)
      return 1;
    if (i == 1)
      return 0;
    rest = rest_tmp;
  }
  return 0;
}

set_quest(q) {
  if (!q)
    return;
  if (query_quests(q))
    return 0;
  if (quests == 0)
    quests = q;
  else
    quests = quests + "#" + q;
  return 1;
}

time_out() {
  write("Time out\n");
  end_session();
}

who() {
  object list;
  int i, nr;

  list = users();
  write("Client    Player\n");
  write("---------------------------------------------------------------------------\n");
  while(i<sizeof(list)) {
    string sh;
    write(list[i]->query_client_version()+"  ");
    sh = list[i]->short();
    if (sh == 0 && level >= 20)
      write("(" + list[i]->query_real_name() + ")\n");
    else if (sh) {
      write(sh + "\n");
      nr += 1;
    }
    i += 1;
  }
  write("\n");
  return 1;
}

static cd(str) {
  if (!str) {
    current_path = "players/" + name;
  } else {
    str = advance(str);
    if (!str || (str != "players/" + name && file_size("/" + str + "/ ") != -2)) {
      write("Illegal path.\n");
      return 1;
    }
    current_path = str;
  }
  if (current_path == ".") {
    current_path = "";
  }
  write("/" + current_path + "\n");
  return 1;
}

#define CHUNK 20

static string more_file;        /* Used by the more command */
static int more_line;

more(str) {
  if (!str)
    return 0;
  more_file = str;
  more_line = 1;
  if (cat(more_file, more_line, CHUNK) == 0) {
    write("No such file\n");
    return 1;
  }
  input_to("even_more");
  write("More: (line " + (CHUNK + 1) + ") ");
  return 1;
}

static even_more(str) {
  if (str == "" || str == "d")
    more_line += CHUNK;
  else if (str == "q") {
    write("Ok.\n");
    return;
  } else if (str == "u") {
    more_line -= CHUNK;
    if (more_line < 1)
      more_line = 1;
  }
  if (cat(more_file, more_line, CHUNK) == 0) {
    more_file = 0;
    write("EOF\n");
    return;
  }
  write("More: (line " + (more_line + CHUNK) + ") ");
  input_to("even_more");
}

pwd() {
  write("/" + current_path + "\n");
  return 1;
}

makedir(str) {
  if (!str)
    return 0;
  if (mkdir(str))
    write("Ok.\n");
  else
    write("Fail.\n");
  return 1;
}

removedir(str) {
  if (!str)
    return 0;
  if (rmdir(str))
    write("Ok.\n");
  else
    write("Fail.\n");
  return 1;
}

query_path() {
  return current_path;
}

summon(str) {
  object ob, env;
  int intmon,resmon;
  string realm, msg;

  if (!str || str == "") {
    write("Summon who?\n");
    return 1;
  }
  ob = find_player(str);
  if (!ob) ob = find_living(str);
  if (!ob) {
    write("No such living thing.\n");
    return 1;
  }
  tell_object(ob, "You are magically summoned somewhere.\n");
/* tell_room(environment(myself), ob->query_name() + " " + msg + ".\n"); */
  ob->magic_move(environment(myself));
  return 1;
}

teleportt(dest) {
  string realm;
  object ob;
  if (!dest) {
    write("Goto where ?\n");
    return 1;
  }
  ob = find_player(dest);
  if (!ob) ob = find_living(dest);
  if (level < 20) spell_points -= 30;
  if (ob) {
    ob = environment(ob);
    if (level < 20)
      if (!valid_teleport(ob)) return 1;
    dest = file_name(ob);
  }
  else if (level < 20) {
    write("No such living thing.\n");
    return 1;
  }
  if (!ob) {
    VALID_OK = 1;   /* horrible Genesis kludge to fix teleport */
    if (file_size(crea_ext(dest)) < 0 || !(dest = valid_read(dest, ARCH))) {
      write("Invalid file name.\n");
      return 1;
    }
  }
  magic_move(dest);
  return 1;
}

finger(str) {
  if (!str) {
    write("Finger who?\n");
    return 1;
  }
  call_other("obj/finger_daemon", "finger", str);
  return 1;
}

find(str) {
  object ob;

  if (!str) {
    write("Find who/what?\n");
    return 1;
  }
  if (level < 20) {
    if (spell_points < 10) {
      write("Too low on spellpoints.\n");
      return 1;
    }
    if (random (100) > intel ) {
      write ("You fail to find object.\n");
      spell_points -= 5;
      return 1;
    }
  }
  ob = find_player(str);
  if (!ob) ob = find_living(str);
  if (!ob) ob = find_object(str);
  if (level < 20) spell_points -= 15;
  if (!ob) {
    write(str + " not found.\n");
    return 1;
  }
  ob = environment(ob);
  if (!ob) {
     write("Sorry, can't find "+str+".\n");
     return 1;
  }
  write(call_other(ob,"short",0)+".\n");
  write("Teleporting/summoning info:\n");
  if (level >= 20) write("Realm : " + ob->realm() + "\n");
  return 1;
}

query_planet() {
  return planet;
}

serve_rqs(str) {
  write(call_other(str,"define")+"@RFS");
  return 1;
}

serve_rqo(str) {
  write(call_other(str,"define")+"@RFS");
  return 1;
}

serve_ver(str) {
  if (strlen(str) != 8) {
    write("Version string rejected!\n");
    return 1;
  }
  client_version = str;
  call_other("obj/version","version_check",client_version);
  return 1;
}

take_a_new_look() {
  call_other(environment(this_object()),"contents");
  return 1;
}

query_client_version() {
  return client_version;
}

/*
 * Some code to handle ghosts...
 * Used to be done with changing graph(), but changing
 * idstr is a whole lot better.
 * To do this, the define func in xthing is replaced
 */
query_desc() {
  if (ghost)
    return("@VIOobj/ghost."+sendhex(pos)+sendhex(depth));
  else
    return("@VIO"+idstr+"."+sendhex(pos)+sendhex(depth));
}

query_ghost() {
  return ghost;
}

levels() {
  call_other("system/levels","list_levels");
  return 1;
}
/*
2
3
4
5
6
7
8
9
0
*/
