/*
 * env_master.c
 * Envelope master object.
 *
 * By Andy Pimentel, 20/1/92
 * Email: pimentel@fwi.uva.nl
 */

#include "post.h"

inherit "lib/stdlib";

static status stamp, filled, closed;
static string addressee, cc_list;
static int nr_cc;
static int pos,depth;
static string idstr;
static string main_idstr;

short()
{
  string s;

  if (closed)
    s = "A closed ";
  else
    s = "An ";
  s += "envelope";
  if (addressee)
    s += " for " + UP(addressee);
  if (stamp == 1)
    s += " with a fresh stamp on it";
  if (stamp == 2)
    s += " with a used stamp on it";
  return s;
}

long()
{
   write("Using all your Sherlock Holmes skills you derive some");
   write("stunning facts:\nYou see an evelope");
   if (closed)
      write(", it's closed.\n");
   else
       write(".\n");
   if (stamp)
      write("There is a stamp stuck on the surface of it.\n");
   if (stamp == 1)
      write("The stamp has not been used yet.\n");
   if (stamp == 2)
      write("The stamp has been stenciled by 'Postal system Earth'.\n");
   if (addressee) 
   {
      write("The envelope has been addressed to:\n       " + 
 	     UP(addressee) + ".\n");
      write("       Robin.lysator.liu.se\n");
      write("       Internet\n");
      if (cc_list) 
         write("Carbon copies go to: " + format_string(cc_list, 22));
   } else
     write("The envelope has no addressee on it.\n");
   if (!addressee && !stamp && !filled)
   {
      write("You can \"write envelope\" for putting an address on it.\n");
      write("An other option is to \"affix stamp\".\n");
   }
}

id(str)
{
   return str == "envelope" || str == S_ENVELOPE_ID;
}

graph() 
{
   return "03040F75857D8F8E8D888203077D8F81878E8D030F7E8F82898C8D";
}

get()
{
   return 1;
}

query_weight() { return 1; }

reset(arg)
{
  resetx(arg);
  stamp = 0;
  addressee = 0;
  filled = 0;
  cc_list = 0;
  nr_cc = 0;
  closed = 0;
}

init()
{
   add_action("write_addressee", "write");
   add_action("fetch_stamp", "affix");
   add_action("close", "close");
   add_action("open", "open");
}

get_envelope(n)
{
   object o;

   if (!me()) return 0;
   for (o = first_inventory(me()); o; o = next_inventory(o))
      if (o->id(S_ENVELOPE_ID) && --n == 0) return o;
   return 0;
}

write_addressee(str)
{
   object o;
   int nr;

   if (str) if (sscanf(str, "%s %d", str, nr) != 2) nr = 1;
   if (!id(str)) 
   {
      notify_fail("Write what?\n");
      return 0;
   }
   if (nr <= 0) nr = 1;
   if (stamp == 2 && nr == 1)
   {
      write("The stamp has been used already...reusage of the envelope is impossible.\n");
      return 1;
   }
   else if (nr != 1)
   {
      if (o = get_envelope(nr))
         o->write_addressee("envelope");
      else
	 write("No such envelope.\n");
      return 1;
   }
   if (addressee)
      write("You erase the old data on the envelope...\n");
   write("Send envelope to whom? : ");
   input_to("get_addressee");
   if (stamp)
     idstr = main_idstr+"__";
   else
     idstr = main_idstr+"_";
   return 1;
}

get_addressee(str)
{
   if (str) str = DOWN(str);
   if (!str || !player_exists(str))
   {
      write("This player does not exist.\n");
      write("Send envelope to whom? : ");
      input_to("get_addressee");
      return 1;
   }
   addressee = str;
   write("Carbon copies? (Y/N) : ");
   input_to("ask_cc");
}

ask_cc(str)
{
   if (str) str = UP(str);
   if (!str || (str != "N" && str != "Y"))
   {
      write("Carbon copies? (Y/N) : ");
      input_to("ask_cc");
      return 1;
   }
   if (str == "N")
   {
      write("Ok.\n");
      return 1;
   }
   else
   {
      write("Just press enter to end.\n");
      write("CC to: ");
      input_to("get_cc");
   }
}

get_cc(str)
{
   if (!str || str == "" || nr_cc == MAX_CC)
   {
      if (nr_cc == MAX_CC)
	 write("Maximum number of CC's is " + MAX_CC + ".\n");
      write("Ok.\n");
      return 1;
   }
   str = DOWN(str);
   if (!player_exists(str))
   {
      write("This player does not exist.\n");
      write("CC to: ");
      input_to("get_cc");
      return 1;
   }
   cc_list = add_to_list(str, cc_list);
   nr_cc++;
   write("CC to: ");
   input_to("get_cc");
}

flush_data(letter)
{
   if (!letter || !objectp(letter)) return;
   letter->set_addressee(addressee);
   letter->set_cc_list(cc_list);
   return;
}

fetch_data(letter)
{
   if (!letter || !objectp(letter)) return;
   addressee = letter->query_addressee();
   cc_list = letter->query_cc_list();
   closed = 1;
   stamp = 2;
   filled = 1;
   return;
}


fetch_stamp(str)
{
   object s;

   if (str != "stamp")
   {
      notify_fail("Affix what?\n");
      return 0;
   }
   if (stamp)
   {
      write("There's a stamp on the envelope already.\n");
      return 1;
   }
   if (!(s = present(S_STAMP_ID, this_player())))
   {
       write("You don't have a stamp!\n");
       return 1;
   }
   move_object(s, environment(this_player()));
   destruct(s);
   stamp = 1;
   idstr = main_idstr+"__";
   write("Stamp affixed.\n");
   return 1;
}

can_put_and_get() { return !closed; }

add_weight(n)     
{
   if (inlist(query_verb(), ({"get", "take"})) != -1)
   {
      filled = 0;
      return 1;
   }
   if (filled)
      return 0;
   filled = 1;
   return 1;
}

close(str)
{
   if (!id(str))
   {
      notify_fail("Close what?\n");
      return 0;
   }
   if (!filled)
   {
      notify_fail("The envelope isn't filled!\n");
      return 0;
   }
   if (closed)
   {
      notify_fail("The envelope has been closed already.\n");
      return 0;
   }
   closed = 1;
   write("You close the envelope.\n");
   return 1;
}

open(str)
{
   if (!id(str))
   {
      notify_fail("Open what?\n");
      return 0;
   }
   if (!closed)
   {
      notify_fail("The envelope is already open.\n");
      return 0;
   }
   closed = 0;
   write("You opened the envelope.\n");
   return 1;
}

query_addressee() { return (addressee) ? UP(addressee) : 0; }
query_stamp()     { return stamp; }
query_filled()    { return filled; }
query_closed()    { return closed; }

/*
 * I am unable to do mutiple inheritance with 3.0  -o , so I
 * manualy include xthing here, and rename some funcs./
 */

resetx(arg) {
  string tmp,instance;

  if (!arg) {
    tmp = file_name(this_object());
    /* Assume all instances look alike, and therefore remove the nr */
    sscanf(tmp,"%s#%s",idstr,instance);
    main_idstr = idstr;
    pos = random(16);
    depth = random(8);
  }
}

locate(p,d) {
  depth = d;
  pos = p;
}

query_pos() {
  return pos;
}

query_depth() {
  return depth;
}

query_desc() {
  return("@VIO"+idstr+"."+sendhex(pos)+sendhex(depth));
}

define() {
  if (!idstr)
    return("@DFO"+file_name(this_object())+"."+graph());
  return("@DFO"+idstr+"."+graph());
}

query_idstr() {
  return(idstr);
}

/*
 * This is a patch! The following code should be available from
 * the parser. Since standard parsers do not provide the sendhex
 * parser call, it's included here in LPC.
 */

makehex(digit) {
  switch(digit) {
    case 0 : return "0";
    case 1 : return "1";
    case 2 : return "2";
    case 3 : return "3";
    case 4 : return "4";
    case 5 : return "5";
    case 6 : return "6";
    case 7 : return "7";
    case 8 : return "8";
    case 9 : return "9";
    case 10: return "A";
    case 11: return "B";
    case 12: return "C";
    case 13: return "D";
    case 14: return "E";
    case 15: return "F";
  }
}

sendhex(byte) {
  int lo, hi;

  hi = byte / 16;
  lo = byte - 16*hi;
  return(makehex(hi)+makehex(lo));
}

purge() {
  object o;
  shout("@PUR"+idstr+".");
  write("@PUR"+idstr+".");
  o = first_inventory(environment(this_object()));
  while (o) {
    if (living(o))
      command("take_a_new_look",o);
    o = next_inventory(o);
  }
}

/*
 * Some objects have instances that do not look alike.
 * E.g. players look different from eachother.
 * If this is the case, then the idstr must include the
 * instance nr.
 */
get_unique_idstr() {
  idstr = file_name(this_object());
}
