/*
 * off_master.c
 * Master object for a post-office.
 *
 * By Andy Pimentel, 21/1/92
 * Email: pimentel@fwi.uva.nl
 */

#include "post.h"

inherit "room/xroom";

int nr_mess;
int last_read;
int last_arrived;
string email_addr;

init()
{
   ::init();
   add_action("mail", "mail");
   add_action("receive", "receive");
   add_action("buy", "buy");
   add_action("mail", "post");
   add_action("fax","fax");
}

reset(arg) {
  ::reset(arg);
}

mail(str)
{
   object env, letter;

   if (str != "letter" && str != "envelope")
   {
      notify_fail("Mail what?\n");
      return 0;
   }
   if (!(env = present(S_ENVELOPE_ID, this_player())))
   {
      write("You don't have a envelope.\n");
      return 1;
   }
   if (!env->query_filled())
   {
      write("The envelope is empty!\n");
      return 1;
   }
   if (!env->query_addressee())
   {
      write("No address on the envelope...how should we deliver this?\n");
      return 1;
   }
   if (!env->query_closed())
   {
      write("Envelope isn't closed...it should be for privacy reasons.\n");
      return 1;
   }
   if (!env->query_stamp())
   {
      write("There is no stamp on the envelope!\n");
      return 1;
   }
   if (!(letter = present(S_LETTER_ID, env)))
   {
      write("Error: no paper inside envelope!\n");
      write("Please report this bug!\n");
      return 1;
   }
   env->flush_data(letter);
   letter->flush_data();
   move_object(letter, environment(this_player()));
   destruct(letter);
   write("A letter for " + env->query_addressee() + " posted.\n");
   move_object(env, environment(this_player()));
   destruct(env);
   return 1;
}

fax(str) {
  object p;

  if (!str) {
    write("Who do you want to fax it to?\n");
    return 1;
  }
  if (!(p = present(S_LETTER_ID, this_player()))) {
    write("You have no paper that you can fax!\n");
    return 1;
  }
  if (!restore_object("users/"+str)) {
    write("no such player.\n");
    return 1;
  }
  if (!email_addr) {
    write("That player does not have a fax machine... sorry!\n");
    return 1;
  }
  write("@BOM"+email_addr+"$"+p->query_message()+"@EOM");
  write("Please, don't forget to type a 'Y' in your xterm window!\n");
  return 1;
}


query_mail()
{
   if (!restore_object(POST_DIR + this_player()->query_real_name()))
      return 0;
   return nr_mess;
}

receive(str)
{
   int n, i;
   object env, letter;

   if (str != "mail")
   {
      notify_fail("Receive what?\n");
      return 0;
   }
   if (!(n = query_mail()))
   {
      write("There's no mail for you...sorry!\n");
      return 1;
   }
   for(i = 0; i < n && i < MAX_LETTERS_ONCE; i++)
   {
      letter = clone_object(POST_BASE + "paper");
      env = clone_object(POST_BASE + "envelope");
      if (!letter->fetch_data(this_player()->query_real_name()))
      {
	 write("Corrupted mail-file! Some letters not received.\n");
	 destruct(letter);
	 destruct(env);
	 return 1;
      }
      env->fetch_data(letter);
      move_object(letter, env);
      if (transfer(env, this_player()))
      {
	 write("You are unable to carry the letter.\n");
	 transfer(env, environment(this_player()));
      }
   }
   write("You received " + i + " letter");
   (i == 1) ? write(".\n") : write("s.\n");  
   if (i == MAX_LETTERS_ONCE)
   {
      write("There are more letters left, but you couldn't get them\n");
      write("all at once. Do \"receive mail\" again to continue.\n");
   }
   return 1;
}
  
buy(str)
{
   object o;

   if (str != "stamp" && str != "envelope" && str != "paper")
   {
      notify_fail("Buy what?\n");
      return 0;
   }
   if (!this_player()->query_money())
   {
      write("You don't have one credit.\n");
      return 1;
   }
   this_player()->add_money(-1);
   if (str == "stamp")
      o = clone_object(POST_BASE + "stamp");
   else if (str == "envelope")
      o = clone_object(POST_BASE + "envelope");
   else
      o = clone_object(POST_BASE + "paper");
   if (transfer(o, this_player()))
   {
      write("You are unable to carry the " + str + ".\n");
      transfer(o, environment(this_player()));
   }
   write("You bought one " + str + ".\n");
   return 1;
}
