/*
 * p_master.c
 * Paper master object.
 *
 * By Andy Pimentel, 20/1/92
 * Email: pimentel@fwi.uva.nl
 */

/*
  minor changes made by Orion 24-3-1992
                                           */
#include "post.h"

inherit "lib/stdlib";

static string message, subject, addressee, cc_list;
static status written;
string body;
int nr_mess;
int last_read;
int last_arrived;

static string idstr, main_idstr;
static int pos,depth;

id(str) { return str == "paper" || str == "letter" || str == S_LETTER_ID; }

short() { return (written) ? "A letter" : "An empty piece of paper"; }

long()
{
   if (written)
   {
      write("A written letter.\n");
   }
   else
   {
      write("An empty piece of paper.\n");
      write("You can write a letter on it, to send mail to other players.\n");
      write("With \"write letter\" you will start writing the letter.\n");
      write("Other commands available: \"read letter\" and \"dump letter\"\n");
      write("destroy the letter.\n");
   }
}

reset(arg)
{
   resetx(arg);
   if (arg) return;
   written = 0;
   cc_list = 0;
   addressee = "";
   message = "";
   subject = "";
   body = "";
   nr_mess = 0;
   last_read = 0;
   last_arrived = 0;
}

init()
{
   add_action("write_letter", "write");
   add_action("read", "read");
   add_action("dump", "dump");
}

get_letter(n)
{
   object o;

   if (!me()) return 0;
   for (o = first_inventory(me()); o; o = next_inventory(o))
      if (o->id(S_LETTER_ID) && --n == 0) return o;
   return 0;
}

write_letter(str)
{
   object o;
   int nr;

   if (str) if (sscanf(str, "%s %d", str, nr) != 2) nr = 1;
   if (!id(str)) 
   {
      notify_fail("Write what?\n");
      return 0;
   }
   if (nr <= 0) nr = 1;
   if (written)
   {
      if (nr == 1)
         write("This letter can only be written once...\n");
      else 
      {
	 if (o = get_letter(nr)) 
	    o->write_letter("letter");
	 else
	    write("No such letter.\n");
      }
      return 1;
   }
   write("Subject: ");
   input_to("get_subject");
   return 1;
}

get_subject(str)
{
   subject = (str) ? str : "";
   write("Write your message and end with \"**\".\n");
   write("Type \"~a\" to abort.\n");
   write(">>> ");
   input_to("get_message");
   return 1;
}

get_message(str)
{
   if (str == "~a")
   {
      write("Writing aborted.\n");
      return 1;
   }
   if (str == "**")
   {
      if (message == "")
      {
	 write("Warning: message body is empty!\n");
	 write("Continue? (Y/N) : ");
	 input_to("cont_body");
	 return 1;
      }
      else
      {
	 written = 1;
         idstr = main_idstr + "_";
         message = "From: " + UP(this_player()->query_real_name()) + ".\n" +
	       	   "Date: " + ctime(time()) + ".\n" + "Subject : " +
		   subject + ".\n\n" + message;
	 write("Letter written.\n");
	 return 1;
      }
   }
   message += str + "\n";
   write(">>> ");
   input_to("get_message");
}

cont_body(str)
{
   if (str) str = UP(str);
   if (!str || (str != "N" && str != "Y"))
   {
      write("Continue? (Y/N) : ");
      input_to("cont_body");
      return 1;
   }
   if (str == "N")
   {
      write("Writing aborted.\n");
      return 1;
   }
   else
   {
       written = 1;
       idstr = main_idstr + "_";
       message = "From: " + UP(this_player()->query_real_name()) + ".\n" +
		 "Date: " + ctime(time()) + ".\n" + "Subject : " +
		 subject + ".\n\n" + message;
       write("Letter written.\n");
       return 1;
   }
}

flush_data()
{
   int n;
   string name, add_body, cc_temp;

   if (!addressee) return;

   cc_temp = (cc_list) ? implode(cc_list, "@@\n") + "@@\n" : ""; 
   add_body = addressee + "#" + cc_temp + message + "@--@\n";
   name = addressee;

   do 
   {
       restore_object(POST_DIR + name);
       nr_mess++;
       body += add_body;
       last_arrived=time();
       save_object(POST_DIR + name);
       if (cc_list && n < sizeof(cc_list))
       {
	  name = cc_list[n++];
	  body = "";
	  nr_mess = 0;
       }
       else name = 0;
    } while (name);

    return;
}

fetch_data(name)
{
   string tmp;
   int i;

   restore_object(POST_DIR + name);
   nr_mess--;
   if (sscanf(body, "%s#%s", addressee, body) != 2) return 0;
   if (sscanf(body, "%s@@\nFrom%s", tmp, body) == 2)
   {
      cc_list = explode(tmp, "@@\n");
      body = "From" + body;
   }
   i = sscanf(body, "%s@--@\n%s", message, body);
   if (i == 1) body = "";
   else if (i != 2) return 0;
   last_read=time();
   save_object(POST_DIR + name);
   written = 1;
   idstr = main_idstr + "_";
   return 1;
}

read(str)
{
   object o;
   int nr;

   if (str) if (sscanf(str, "%s %d", str, nr) != 2) nr = 1;
   if (!id(str))
   {
      notify_fail("Read what?\n");
      return 0;
   }
   if (nr <= 0) nr = 1;
   if (!written && nr == 1)
   {
       write("The letter isn't written yet. The paper is empty.\n");
       return 1;
   }
   else if (nr != 1)
   {
      if (o = get_letter(nr)) 
         o->read("letter");
      else
         write("No such letter.\n");
      return 1;
   }
   write(message);
   return 1;
}

query_mail(str) {
	if(restore_object(POST_DIR+str)) {
		if(last_read<last_arrived) return "You have new mail.\n";
		if (nr_mess >1) return "You have mail.\n";
		return "";
	}
	return "";
}

dump(str)
{
   if (!id(str))
   {
      notify_fail("Dump what?\n");
      return 0;
    }
    write("Are you sure to dump letter? (Y/N) : ");
    input_to("do_dump");
    return 1;
}

do_dump(str)
{
   if (str) str = UP(str);
   if (!str || (str != "Y" && str != "N"))
   {
      write("Are you sure to dump letter? (Y/N) : ");
      input_to("do_dump");
      return 1;
   }
   if (str == "N")
   {
      write("Letter not dumped.\n");
      return 1;
   }
   else
   {
      write("Letter dumped...\n");
      destruct(this_object());
      return 1;
   }
}

get() { return 1; }

query_weight() { return 1; }
query_message() { return message; }

set_addressee(s)  { addressee = s; }
set_cc_list(l)    { cc_list = l; }

query_addressee() { return addressee; }
query_cc_list()   { return cc_list; }

graph() {
  return "01080F7189788A7E88847F8E86898D81917A91";
}

/*
 * I am unable to do mutiple inheritance with 3.0  -o , so I
 * manualy include xthing here, and rename some funcs./
 */

resetx(arg) {
  string tmp,instance;

  if (!arg) {
    tmp = file_name(this_object());
    /* Assume all instances look alike, and therefore remove the nr */
    sscanf(tmp,"%s#%s",idstr,instance);
    main_idstr = idstr;
    pos = random(16);
    depth = random(8);
  }
}

locate(p,d) {
  depth = d;
  pos = p;
}

query_pos() {
  return pos;
}

query_depth() {
  return depth;
}

query_desc() {
  return("@VIO"+idstr+"."+sendhex(pos)+sendhex(depth));
}

define() {
  if (!idstr)
    return("@DFO"+file_name(this_object())+"."+graph());
  return("@DFO"+idstr+"."+graph());
}

query_idstr() {
  return(idstr);
}

/*
 * This is a patch! The following code should be available from
 * the parser. Since standard parsers do not provide the sendhex
 * parser call, it's included here in LPC.
 */

makehex(digit) {
  switch(digit) {
    case 0 : return "0";
    case 1 : return "1";
    case 2 : return "2";
    case 3 : return "3";
    case 4 : return "4";
    case 5 : return "5";
    case 6 : return "6";
    case 7 : return "7";
    case 8 : return "8";
    case 9 : return "9";
    case 10: return "A";
    case 11: return "B";
    case 12: return "C";
    case 13: return "D";
    case 14: return "E";
    case 15: return "F";
  }
}

sendhex(byte) {
  int lo, hi;

  hi = byte / 16;
  lo = byte - 16*hi;
  return(makehex(hi)+makehex(lo));
}

purge() {
  object o;
  shout("@PUR"+idstr+".");
  write("@PUR"+idstr+".");
  o = first_inventory(environment(this_object()));
  while (o) {
    if (living(o))
      command("take_a_new_look",o);
    o = next_inventory(o);
  }
}

/*
 * Some objects have instances that do not look alike.
 * E.g. players look different from eachother.
 * If this is the case, then the idstr must include the
 * instance nr.
 */
get_unique_idstr() {
  idstr = file_name(this_object());
}
