/*
 * Inherit file for rooms in a BSX-MUD. This is how to incorporate it:
 *
 *   inherit "room/xroom";
 *   reset(arg) {
 *     ::reset(arg);
 *     if(arg) return;
 *	 graphic = "xxxxxx";
 *       description = "You are standing on the village green.\n"+
 *                     "You can see some trees here.\n";
 *       items = ({"trees","Some beautiful trees."});
 *       dest_dir = ({ "room/church", "north",
 *                    "room/bridge", "west",
 *                    "room/road", "east",
 *                    "room/alley", "south" });
 *   }
 *
 */

/*
    set_dark & skip_obvious & skip_contents added by Orion
    and no_moves();

    set_dark(0): (default) make it light.
    set_dark(1): (default) make it dark.

    skip_obvious: don't show the obvious exits
    skip_contents: don't show the contents (don't draw them either)
    no_moves() : call it when you don't want a player to be able to micro_move
                 (for instance in init() )

*/

string idstr;
string dest_dir;
string items;
string description;
string graphic;
int skip_obvious;
int skip_contents;

init() {
  int i, n;
  string func;

  if (!dest_dir)
    return;
  n = sizeof(dest_dir);
  if (n > 2) {
    func = dest_dir[2];

    if (!func)
      n = 3;
    else
      if(func[0] == 35)
        n = 3;
      else 
        n = 2;
  }
  else
    n = 2;

  while(i < sizeof(dest_dir)) {
    add_action("move", dest_dir[i + 1]);
    i += n;
  }
}

id(str) {
    int i;
    if (!items)
        return;
    while(i < sizeof(items)) {
        if (items[i] == str)
            return 1;
        i += 2;
    }
    return 0;
}

long(str) {
  int i, n;
  string func;
  object ob;

  if(!set_light(0)) {
    write("It is too dark\n");
    write("@SCEdummies/darkroom.@RFS");
    return 1;
  }
  if(str) {
	if(!items) return;
	i=0;
	while(1<sizeof(items)) {
		if(items[i]=str) {
			write(items[i+1]+"\n");
			return;
		}
		i+=2;
	}
	return;
  }
  if(description) write(description);
  contents();   /* Does basicaly all the graph stuff */
  if(!skip_obvious) {
    if (!dest_dir) {
      write("There are no obvious exits.");
    } else {
    n = sizeof(dest_dir);
    if (n > 2) {
      func = dest_dir[2];
      if (!func)
        n = 3;
      else
       if(func[0] == 35)
         n = 3;
       else 
         n = 2;
    } else
      n = 2;
    i = 1;
    if (sizeof(dest_dir) == n)
      write(" There is one obvious exit:");
    else
      write(" There are " + convert_number(sizeof(dest_dir) / n) +
      " obvious exits:");
    while(i < sizeof(dest_dir)) {
      write(" " + dest_dir[i]);
      i += n;
      if (i == sizeof(dest_dir) - 1)
        write(" and");
      else if (i < sizeof(dest_dir))
        write(",");
    }
    }
    write("\n");
  }
  if(!skip_contents) {
    ob = first_inventory(this_object());
    while (ob) {
      if (ob != this_player())
        if(ob->short()) write(" "+ob->short()+"\n");
      ob = next_inventory(ob);
    }
  }
}

move(str) {
  int i, n;
  string func;

  n = sizeof(dest_dir);
  if (n > 2) {
    func = dest_dir[2];
    if (!func)
      n = 3;
    else
      if (func[0] == 35)
        n = 3;
      else 
        n = 2;
  }
  else
    n = 2;

  while (i < sizeof(dest_dir)) {
    if (query_verb() == dest_dir[i + 1]) {
      if (n == 3 && dest_dir[i + 2] != 0) {
        if (!sscanf(dest_dir[i + 2], "#%s", func))
          return;
        if (call_other(this_object(), func, dest_dir[i]))
          return 1;
      }
      this_player()->move_player(dest_dir[i + 1] + "#" + dest_dir[i]);
      return 1;
    }
    i += n;
  }
}

query_dest_dir() {
  return dest_dir;
}

query_long() {
  return description;
}

/*
 * Convert a number to a word. The array is being created by the
 * standard room/xroom, and shared by all rooms.
 */
string numbers;

convert_number(n) {
  if (!pointerp(numbers))
    numbers = query_numbers();
  if (n > 9)
    return "lot of";
  return numbers[n];
}

query_numbers() {
  return ({"no", "one", "two", "three", "four", "five",
           "six", "seven", "eight", "nine" });
}

reset(arg) {
  if (!arg) {
    idstr = file_name(this_object());
    skip_obvious=0;
    skip_contents=0;
    set_light(1);
  }
}

define() {
  return("@DFS"+idstr+"."+graph());
}

contents() {
  object ob;

  write("@SCE"+idstr+".");
  /* Now list all the present people/things */
if(!skip_contents) {
  ob = first_inventory(this_object());
  while(ob) {
   if(!ob->query_invis()) {
    if(ob->query_desc()) write(ob->query_desc());
   }
    ob = next_inventory(ob);
  }
}
  write("@RFS");
}

/*
 * This routine is to resend the state of a room to all clients.
 * If e.g. an idstr has changed, or if objects (dis)appeared, then
 * call this routine
 */
resend_state() {
  object ob;
  if(!set_light(0)) {
    tell_room(this_object(),"@SCEdummies/darkroom.@RFS");
  } else {

  tell_room(this_object(),"@SCE"+idstr+".");
  /* Now list all the present people/things */
  if(!skip_contents) {
    ob = first_inventory(this_object());
    while(ob) {
      if(!ob->query_invis()) {
      if(ob->query_desc()) tell_room(this_object(),ob->query_desc());
      }
      ob = next_inventory(ob);
    }
  }
  tell_room(this_object(),"@RFS");
  }
}

graph() {
  if (!graphic) return "00";
  return graphic;
}

set_graph(g) {
  graphic = g;
}

set_description(d) {
  description = d;
}

set_dest_dir(d) {
  dest_dir = d;
}

set_dark(arg) {
	if(!arg) {
		set_light(-set_light(0)+1);
	} else {
		set_light(-set_light(0));
	}
}
query_skip_contents() {
	return skip_contents;
}
no_moves() {
	add_action("noway","micro_w");
	add_action("noway","micro_e");
	add_action("noway","micro_s");
	add_action("noway","micro_n");
}
noway() {
	return 1;
}
